/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#line 1 "Fhp6_msite.c"
/* 
  $Author: zaleski $
  $Log: Fast.c,v $
 * Revision 1.2  1994/10/14  08:49:28  zaleski
 * added automatic directory creation.
 *
  $Revision: 1.2 $
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h> 
#include <limits.h>
/* 
--- MACROS ---

We use 7 bit macros and structure, only we collode and proapagate 6 bits only. 

*/

#include "macros_7.h"
#include "macros_gen.h"
#include "parameters_6_msite.h"

/* long random(); */

/* 
The macro below is helpful for writing more concisely 
the propagation step on every boundary
and corner. As input it takes the coordinates 
of the site (ii, jj) and those (ii_dn, jj_dn), etc... 
of the four nearest neighbors on the hexagonal axes.
The coordinates of the four nearest neighbors are computed 
for each case, but the macro stays the same.  
*/ 

   
/* 6 part propagation */ 

#define PROPAGATE(ii,ii_dn,ii_up,jj,jj_dn,jj_up) \
  zword  = (ii) + max_x_word_coord * (jj); \
  zword_up = (ii) + max_x_word_coord * (jj_up); \
  zword_down  = (ii) + max_x_word_coord * (jj_dn);\
  zword_next_right = (ii_up) +  max_x_word_coord * (jj);\
  zword_next_left = (ii_dn) +  max_x_word_coord * (jj); \
  a[zword] = (na[zword] >> 1) | (na[zword_next_left] << 31);\
  b[zword_up] = nb[zword];\
  c[zword_up] = (nc[zword] << 1 ) | (nc[zword_next_right] >> 31);\
  d[zword] =  (nd[zword] << 1) | (nd[zword_next_right] >>  31);\
  e[zword_down] = ne[zword]; \
  f[zword_down] = (nf[zword] >> 1) |  (nf[zword_next_left] <<  31); 

main()
{
  int nx,ny,nz,i,j,k,z,x,y,t,px,py,npart;
  int zword,zlatt,iword;
  int bit_from_left;
  int bit_order;
  int zword_up,  zword_down, zword_next_right, zword_next_left;
  int pg, force_push, force_attempts, attempt, block_len, jblock, address;
  int j1, max_y_block_coord;
  long int ratio;
  int line_xmom[NY], line_ymom[NY], line_npart[NY];
  int count_a,count_b,count_c,count_d,count_e,count_f,count_g,tstart_aver;
  
  unsigned int t1,t2,t3,t4,t5,t6,u1,u2,u3,g1,g2,g3,del,c1,c2,c3;
  FILE *history_file_fd, *outfd, *solidfd;
  unsigned int *a, *b, *c, *d, *e, *f, *g, *s;
  unsigned int *na, *nb, *nc, *nd, *ne, *nf, *ng, *ns; 
  int  word_len, max_x_word_coord,n_words,seed;
  float dens;
  unsigned char *lat_out, *solid;
  unsigned int particle,*latt;

/* Counting momentum for measurement of viscsotiy */ 

  double av_force,avxmom[NY], avdens[NY]; 
  int ncount, ncount_force , nexp , i_exp; 
  float force_int, forcing_rate, minf, maxf; 

/* Perform some basic initialisation */

  nx=NX;
  ny=NY;
  nz=nx*ny;

/* 
  some machines have different sizes for integers, so we compute the 
  lenght of the words we are goping to use 
*/

  word_len=8*sizeof(int);
  printf("word_len=%d\n",word_len); 
  n_words=nz/word_len;
  max_x_word_coord=nx/word_len;
  dens=DENSITY;
  

#ifdef FORCING_RATE
  forcing_rate = FORCING_RATE;
  printf("forcing_rate=%e\n",forcing_rate);
  block_len = word_len;
  max_y_block_coord = ny/block_len;
  force_int = forcing_rate * block_len*word_len;
  force_push = (int ) ( force_int ); 
  if((force_int < 1. ) && (force_int > 0.))
	{
	  force_push = 1; 
	  printf("Low forcing rate \n");
	}
  if(forcing_rate <= 0.) 
	{
	  fprintf(stderr,"forcing_rate is <= 0 , undefine it in source to improve performance\n");
	  exit(-1);
	}
/*       force_attempts = 64 * (int)( force_push /((1. - dens) * dens)); */ 
      force_attempts = word_len * block_len; 
      printf("force_push=%d\nforce_attempts=%d\n",force_push,force_attempts);

#endif    /* #ifdef FORCING_RATE) */
  
/* Initialize output and some files for graphics */
  
  if ((outfd = fopen("dumpfile","w")) == NULL)
    {
      fprintf(stderr,"Could not open file -- dumpfile --. \n");
      exit(1); 
    }
  
  history_file_fd = fopen("history","w");
  fprintf(history_file_fd,"in=dumpfile\nn1=%d\nn2=%d\ncolfile=dpartercol\n",nx,ny);
  fclose(history_file_fd);
  
/* 
    We may either use a fixed seed, to obtain each time the same
    pseudo random behavior of the code, 
    or initialize random number generator differently for each run 
*/

#ifdef SEED  
  seed = SEED;
#else
  seed = time(0); 
#endif

  srandom(seed);
  /* Test collision functions */ 
  
  test_collision(); 

  /* Reserve memory space */
  
  a  =    (unsigned int *) malloc(n_words*sizeof(unsigned  int)); 
  b  =    (unsigned int *) malloc(n_words*sizeof(unsigned  int)); 
  c  =    (unsigned int *) malloc(n_words*sizeof(unsigned  int)); 
  d  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  e  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  f  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  g  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  s  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  
  na  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  nb  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  nc  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  nd  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  ne  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  nf  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  ng  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  ns  =    (unsigned int *) malloc(n_words*sizeof(unsigned int)); 
  
  latt =   (unsigned int *) malloc(nz*sizeof(unsigned int)); 
  
  /* Notice latt is an int here */ 
  
  lat_out = (unsigned char *) malloc(nz*sizeof(char)); 
  solid =   (unsigned char *) malloc(nz*sizeof(char));
  
  /* INITIALIZE VELOCITY PROFILES and COUNTING */

  tstart_aver = TSTART_AVER;
  for(j=0; j< ny; j++)
    {
      avxmom[j] = 0.;
      avdens[j] = 0.; 
      ncount = 0;
      ncount_force=0; 
    }
  av_force=0.; 


  /* INITIAL CONDITIONS */


  
  /* Initialize the lattice with density dens  */
  
  
  for(z=0;z<nz;z++)
    {
      latt[z] = 0;
    }
  
  for(z=0;z<nz;z++)
    {
      for(k=0; k<7; k++)
	{
/* 
   The variable particle will be equal to 1 with probability dens 
   and to 0 with probability 1 - dens 
*/ 
	  particle= (int) (((float) random())/((float) LONG_MAX + 1.)
			   + dens);    
	  if(particle)
	    {
	      latt[z] |= (1 << k); 
	    }
	}
    }
  /* Read a file containing data about solid sites */
  
	if((solidfd = fopen("solid","r")) != NULL) 
	  {
	    i=fread((char *) solid, sizeof(char), nz*sizeof(char),solidfd);
	  }
	else
	  {
	    fprintf(stderr,"No solid site configuration found\n");
	    fprintf(stderr,"Continuing normal operation without solid sites\n");
	    for(z=0;z<nz;z++)
	      solid[z]=0;
	  }
  
  /* Translate to hex coordinates and place into single site words latt[z] */ 
  /* Solid is in cartesian coordinates and latt in HEX coordinates */
  
  for(i=0;i<nx;i++)
    for(j=0;j<ny;j++)
      {
	x = X_HEX_TO_CART(i,j);
	y = Y_HEX_TO_CART(i,j); 
	if(solid[x + nx*y]) latt[i + nx*j] |= S; 
      }
  
  printf("initial conditions"); test_cons(latt,&nz,&px,&py,&npart); 
  
  /* Translate initial conditions from the initial data format to 
     multispin format */
  
  /* Clear all bits */
  
  for(zword = 0 ; zword < n_words; zword++)
    {
      a[zword] = 0;
      b[zword] = 0;
      c[zword] = 0;
      d[zword] = 0;
      e[zword] = 0;
      f[zword] = 0;
      g[zword] = 0;
      s[zword] = 0;
      
      na[zword] = 0;
      nb[zword] = 0;
      nc[zword] = 0;
      nd[zword] = 0;
      ne[zword] = 0;
      nf[zword] = 0;
      ng[zword] = 0;
      ns[zword] = 0;
    }
  
  for(j=0;j < ny; j++)
    {
      for(i=0;i < nx; i++)
	{
	  zlatt = i + nx*j;
	  iword = i / word_len;
	  zword = iword + max_x_word_coord * j; 
	  /* Bit counted from left of word */

	  bit_from_left = i - iword*word_len; 

	  /* BIt order in word (counted from right) */

	  bit_order = word_len - 1 - bit_from_left;

	  /* Create multisite coding */

	  if(AP(latt[zlatt]))  a[zword] |= (1 << bit_order); 
	  if(BP(latt[zlatt]))  b[zword] |= (1 << bit_order); 
	  if(CP(latt[zlatt]))  c[zword] |= (1 << bit_order); 
	  if(DP(latt[zlatt]))  d[zword] |= (1 << bit_order); 
	  if(EP(latt[zlatt]))  e[zword] |= (1 << bit_order); 
	  if(FP(latt[zlatt]))  f[zword] |= (1 << bit_order); 
	  if(GP(latt[zlatt]))  g[zword] |= (1 << bit_order); 
	  if(SP(latt[zlatt]))  s[zword] |= (1 << bit_order); 
	}
    }
  
  /* MAIN LOOP */
  
  for(t=0;t<TMAX;t++)
    {
/* The forcing algorithm */
#ifdef FORCING_RATE
      ratio = (long int) (((float) LONG_MAX + 1.)/(word_len*block_len));
      address = (int) (((long int) random())/ratio); 
      ncount_force++; 
      for(jblock=0;jblock < max_y_block_coord ; jblock++)
	{
	  for(iword=0;iword < max_x_word_coord; iword++)
	    {
	      pg =0; k = 0;
	      if(force_int < 1. ) 
		{
/* Draw a random integer between 0 and word_len*block_len */ 
         	  attempt = (int) (((float) random())/((float) LONG_MAX + 1.)
				   + force_int);    
	          pg = 1 - attempt;
		  if(force_push != 1) 
		    {
		      printf("error on force_push !");
		      exit(-1);
		    }		
		}  
	      while(( pg < force_push ) &&  (k < force_attempts) ) 
		{ 
		  if( ( attempt != 1 ) && (force_int < 1. ) ) 
		    {
		      printf("error on attempt !");
		      exit(-1);
		    }		
/* Try next site */ 
		  address = (address +1)%(word_len*block_len); 

/* Compute the coordinate of the selected word */
		  j1 = address/block_len; 
		  j = jblock*block_len + j1; 
		  bit_order = wordlen -1 - (address - j1*block_len); 
		  zword = iword + max_x_word_coord * j; 

/* test if the site is a candidate for reversing the a particle */

		  if( ((a[zword] & ~(d[zword])) & ( 1 << bit_order )) != 0 )
		    {
		      a[zword]  = a[zword] & ~(1 << bit_order); 
		      d[zword] |= (1 << bit_order); 
		      address = (int) (((long int) random())/ratio); 
		      if(((s[zword] & (1<<bit_order)) == 0 ) && (t>tstart_aver)) 
			{
			  av_force += 2.; 
			}
		      pg++; 
		    }
		  k++;
		}
	      if(pg < force_push)
		{
		  printf(" Failed to apply forcing after pg %d, k %d \n in block iword = %d   jblock = %d \n", pg, k, iword, jblock);
		}
	    }
	}

#endif  /* FORCING_RATE */

      if(t&1) 
	{
	  for(j=0;j<ny;j++)
	    {
	      for(iword=0;iword < max_x_word_coord; iword++)
		{
		  zword = iword + max_x_word_coord * j; 
		  collision_6_right(a[zword],b[zword],c[zword],
d[zword],e[zword],f[zword],g[zword],na+zword,nb+zword,nc+zword,
nd+zword,ne+zword,nf+zword,ng+zword,s[zword]);
		}
	    }      
	}
      else
	{
	  for(j=0;j<ny;j++)
	    {
	      for(iword=0;iword < max_x_word_coord; iword++)
		{
		  zword = iword + max_x_word_coord * j; 
		  collision_6_left(a[zword],b[zword],c[zword],
d[zword],e[zword],f[zword],g[zword],na+zword,nb+zword,nc+zword,
nd+zword,ne+zword,nf+zword,ng+zword,s[zword]);
		}
	    }      
	}
	  

/* PROPAGATION */

      for(j=1;j < ny-1; j++)
	{
	  for(iword=1;iword < max_x_word_coord-1; iword++)
	    {
	      PROPAGATE(iword,iword-1,iword+1,j,j-1,j+1)
	    }
	      /* Treat border lines with wrap around condition  */
	    PROPAGATE(0,max_x_word_coord-1,1,j,j-1,j+1)
            PROPAGATE(max_x_word_coord-1,max_x_word_coord-2,0,j,j-1,j+1)
	  }
	for(iword=1;iword < max_x_word_coord-1; iword++)
	  {
	    PROPAGATE(iword,iword-1,iword+1,0,ny-1,1)
	    PROPAGATE(iword,iword-1,iword+1,ny-1,ny-2,0)
	  }
	  /* Treat corners */
	  PROPAGATE(0,max_x_word_coord-1,1,0,ny-1,1)
	  PROPAGATE(max_x_word_coord-1,max_x_word_coord-2,0,0,ny-1,1)
	  PROPAGATE(0,max_x_word_coord-1,1,ny-1,ny-2,0)
          PROPAGATE(max_x_word_coord-1,max_x_word_coord-2,0,ny-1,ny-2,0)
		  

	  /* Measure velocity profile */
#ifdef COUNTING
if(t > tstart_aver)
	    {
	      int all_npart, all_xmom, all_ymom; 
	      
	      all_npart=all_xmom = all_ymom=0;
	      for(j=0;j<ny;j++) 
		{
		  line_xmom[j] = line_ymom[j] = line_npart[j] = 0; 
		  count_a = count_b = count_c = count_d = count_e = count_f = count_g = 0; 
		  for(iword=0;iword < max_x_word_coord; iword++)
		    {
		      zword = iword + max_x_word_coord * j; 
		      count_a +=  NUMBER_IN_WORD(a[zword]);
		      count_b +=  NUMBER_IN_WORD(b[zword]);
		      count_c +=  NUMBER_IN_WORD(c[zword]);
		      count_d +=  NUMBER_IN_WORD(d[zword]);
		      count_e +=  NUMBER_IN_WORD(e[zword]);
		      count_f +=  NUMBER_IN_WORD(f[zword]);
		      count_g +=  NUMBER_IN_WORD(g[zword]);
		    }
		  line_xmom[j]  = 2*(count_a - count_d) + count_b + count_f - count_c - count_e; 
		  line_ymom[j]  = count_b  + count_c - count_f  - count_e; 
		  line_npart[j] = count_a + count_b + count_c + count_d + count_e + count_f + count_g; 
/*
		  all_xmom  += line_xmom[j];
		  all_ymom  += line_ymom[j];
		  all_npart += line_npart[j];
*/
		  avxmom[j] += (double) (line_xmom[j]);
		  avdens[j] += (double) (line_npart[j]);
	      }
/*	      printf("all_np=%d all_xm=%d all_ym=%d\n",all_npart,all_xmom,all_ymom); 
*/
	      ncount++;
	    }
/* end if (t > tstart_aver) */ 
#endif /* COUNTING */

	  /* Now print and test conservation every so many steps */
		  
	  if(t%TPRINT == 0 ) 
	    {
	      for(j=0;j < ny; j++)
		{
		  for(i=0;i < nx; i++)
		    {
		      zlatt = i + nx*j;
		      iword = i / word_len;
		      zword = iword + max_x_word_coord * j; 
		      latt[zlatt] = 0; 

		      /* Bit counted from left of word */

		      bit_from_left = i - iword*word_len; 

		      /* BIt order in word (counted from right) */

		      bit_order = word_len - 1  - bit_from_left;

/* Mask multisite word, 
   push bit to the right of multisite word a,b,c ,..., 
   then left to correct position on single site word latt[z] 
*/
		      latt[zlatt]  =  (a[zword] >> bit_order) & 1 ; 
		      latt[zlatt] |= ((b[zword] >> bit_order) & 1 )  << 1 ;
		      latt[zlatt] |= ((c[zword] >> bit_order) & 1 )  << 2 ;
		      latt[zlatt] |= ((d[zword] >> bit_order) & 1 )  << 3 ;
		      latt[zlatt] |= ((e[zword] >> bit_order) & 1 )  << 4 ;
		      latt[zlatt] |= ((f[zword] >> bit_order) & 1 )  << 5 ;
		      latt[zlatt] |= ((g[zword] >> bit_order) & 1 )  << 6 ;
		      latt[zlatt] |= ((s[zword] >> bit_order) & 1 )  << 7 ;
		    }
		}
	      
/* Transfer to cartesian grid */ 
	      
	      for(i=0;i<nx;i++)
		for(j=0;j<ny;j++)
		  {
		    x = X_HEX_TO_CART(i,j);
		    y = Y_HEX_TO_CART(i,j); 
		    lat_out[x + nx*y] = latt[i + nx*j];
		  }
	      printf("t=%d\n ",t); test_cons(latt,&nz,&px,&py,&npart); 
	      fwrite((char *) lat_out, sizeof(char), nz*sizeof(char),outfd);
	    }
    }
/* End of main loop */


#ifdef COUNTING
/* Print some results and measure viscosity for Poiseuille flow experiment */ 
  {
    FILE *profilefd;
    char *dirname, *command, *tmp, *filename;
    float rho, velo[NY], b, prodvfit , fit2, nu_0, u_0 , nu_t, force;
    float umax,cs;

    dirname =  (char *) malloc(80*sizeof(char)); 
    command =  (char *) malloc(80*sizeof(char)); 
    tmp =  (char *) malloc(80*sizeof(char)); 
    filename =  (char *) malloc(80*sizeof(char)); 

/* Print the true density to correct a systematic error in calculations 
  that use dens */

    rho = ((float) npart)/(nx*ny);
    printf("true density = %f\n",rho);
    sprintf(dirname,"Results_Force%.1e",forcing_rate);
    tmp = strcpy(command,"mkdir ");       /* command == "mkdir "  */ 
    system(strcat(command,dirname));      /* command == mkdir dirname */
    strcpy(filename,dirname);
    strcat(filename,"/");
    strcat(filename,"profile");           /* filename ==  dirname/profile */
    printf("open %s \n",filename);
    profilefd = fopen(filename,"w");
    for(j=0;j<ny;j++)
      {
	avxmom[j] = avxmom[j]/((double) (2*ncount*nx));
	fprintf(profilefd,"%e\n",avxmom[j]);
      }
    fclose(profilefd);
    force = av_force/(ncount_force*nx*(ny-2));
    printf("average force %e\n",force);
    printf(" count %d \n",(ncount*nx*(ny-2)));

/* Compute best fit to viscosity */ 

    dens = rho/7.;
    for(j=0;j<ny;j++)
      velo[j] = avxmom[j] / rho;
    b = 0.5;
    prodvfit = fit2 = 0.;
    for(j=1; j < ny-1 ; j++)
      {
	prodvfit += velo[j]*(j - b)*( ny - 1 - b - j);
	fit2 += (j - b) * ( ny - 1 - b - j) * (j - b) * ( ny - 1 - b - j);
      }
    u_0 = prodvfit/ fit2; 
    nu_0 = - 3 * force/ (rho * 8 * u_0) ;
    nu_t = (1./28.)*(1./(dens*(1.-dens)))*(1./(1- 8.*dens*(1-dens)/7.))-1./8.;
    printf("force %e nu_0 %e nu_t %e\n",forcing_rate,nu_0, nu_t);

    umax = u_0 *  (ny/2  - b) * ( ny/2 - 1 - b ) ;
    cs = sqrt(3./7.);
    printf("Mach = %e\n", umax/cs);
  }
#endif
  fprintf(stderr,"Normal exit from Fhp6_msite ! \n");
  return 0;
}


test_cons(latt, nz, px, py, n)  unsigned int *latt;int *nz;int *px; int *py; int *n;
 {
  int z;
  *px= *py= *n = 0;
  for(z=0;z< *nz; z++)
    {
      *px += X_MOM(latt[z]);
      *py += Y_MOM(latt[z]);
      *n  += PARTSUM(latt[z]);
    }
/*  fprintf(stderr," number: %d, x_mom %d, y_mom %d  \n",*n,*px,*py); */
  fprintf(stdout," number: %d, x_mom %d, y_mom %d  \n",*n,*px,*py);

#ifdef DIAGNOSTICS
  *n=0;
  for(z=0;z< *nz; z++)
    *n += AP(latt[z]);
  fprintf(stdout,"  A %d",*n);
  *n=0;
  for(z=0;z< *nz; z++)
    *n += BP(latt[z]);
  fprintf(stdout,"  B %d",*n);
  *n=0;
  for(z=0;z< *nz; z++)
    *n += CP(latt[z]);
  fprintf(stdout," C %d",*n);
  *n=0;
  for(z=0;z< *nz; z++)
    *n += DP(latt[z]);
  fprintf(stdout," D %d",*n);
  *n=0;
  for(z=0;z< *nz; z++)
    *n += EP(latt[z]);
  fprintf(stdout," E %d",*n);
  *n=0;
  for(z=0;z< *nz; z++)
    *n += FP(latt[z]);
  fprintf(stdout,"  F %d",*n);
  *n=0;
  for(z=0;z< *nz; z++)
    *n += GP(latt[z]);
  fprintf(stdout,"  G %d\n",*n);
#endif /* #ifdef DIAGNOSTICS */ 
}
   
#ifdef DIAGNOSTICS
#ifdef FORCING_RATE
bin(i)  i;
unsigned int i; 
{
int k;
  for(k=0; k< 32; k++)
    {
        printf("%d",i%2); 
        i = i/2; 
     }
}
#endif
#endif
/* 6 particle collision operator */ 

collision_6_left(a, b, c, d, e, f, g, na, nb, nc, nd, ne, nf, ng, s)  unsigned int a; unsigned int b;unsigned int c;unsigned int d; unsigned int e; unsigned int f; unsigned int g; unsigned int *na; unsigned int *nb;unsigned int *nc;unsigned int *nd; unsigned int *ne; unsigned int *nf; unsigned int *ng; unsigned int s;
{
   unsigned int t1,t2,t3,t4,t5,t6,u1,u2,u3,g1,g2,g3,del,c1,c2,c3;

/* Exclusive or is ^ */
	      t1 = a^b;
	      t2 = b^c;
	      t3 = c^d;
	      t4 = d^e;
	      t5 = e^f;
	      t6 = f^a;
	      
	      u1 = t1 & t4;
	      u2 = t2 & t5;
	      u3 = t3 & t6;
	      
/* g means pair of opposing particles */ 

	      g1 = u1 & ( b^d );
	      g2 = u2 & ( c^e );
	      g3 = u3 & ( d^f );
	      
	      del = u1 & u2 & u3;
	      
	      /* rotate left */
	      
	      c1 = del | g1 | g3 & ~g2;
	      c2 = del | g2 | g1 & ~g3;
	      c3 = del | g3 | g2 & ~g1;
	      
#ifdef SOLID_SITES
	      *na =((~s) & (c1^a)) |( s & d);
	      *nb =((~s) & (c2^b)) |( s & e);
	      *nc =((~s) & (c3^c)) |( s & f);
	      *nd =((~s) & (c1^d)) |( s & a);
	      *ne =((~s) & (c2^e)) |( s & b);
	      *nf =((~s) & (c3^f)) |( s & c);
#else
	      *na = c1^a;
	      *nb = c2^b;
	      *nc = c3^c;
	      *nd = c1^d;
	      *ne = c2^e;
	      *nf = c3^f;
#endif

 
}

/* 6 particle collision operator */ 


collision_6_right(a, b, c, d, e, f, g, na, nb, nc, nd, ne, nf, ng, s)  unsigned int a; unsigned int b;unsigned int c;unsigned int d; unsigned int e; unsigned int f; unsigned int g; unsigned int *na; unsigned int *nb;unsigned int *nc;unsigned int *nd; unsigned int *ne; unsigned int *nf; unsigned int *ng; unsigned int s;
{
   unsigned int t1,t2,t3,t4,t5,t6,u1,u2,u3,g1,g2,g3,del,c1,c2,c3;

   /* Exclusive or is ^ */
   t1 = a^b;
   t2 = b^c;
   t3 = c^d;
   t4 = d^e;
   t5 = e^f;
   t6 = f^a;
	      
   u1 = t1 & t4;
   u2 = t2 & t5;
   u3 = t3 & t6;
	      
   /* g means pair of opposing particles */ 

   g1 = u1 & ( b^d );
   g2 = u2 & ( c^e );
   g3 = u3 & ( d^f );
	      
   del = u1 & u2 & u3;
	      
   /* odd time step, rotate right */
		 
   c1 = del | g3 | g1 & ~g2;
   c2 = del | g1 | g2 & ~g3;
   c3 = del | g2 | g3 & ~g1;

#ifdef SOLID_SITES
	      *na =((~s) & (c1^a)) |( s & d);
	      *nb =((~s) & (c2^b)) |( s & e);
	      *nc =((~s) & (c3^c)) |( s & f);
	      *nd =((~s) & (c1^d)) |( s & a);
	      *ne =((~s) & (c2^e)) |( s & b);
	      *nf =((~s) & (c3^f)) |( s & c);
#else
	      *na = c1^a;
	      *nb = c2^b;
	      *nc = c3^c;
	      *nd = c1^d;
	      *ne = c2^e;
	      *nf = c3^f;
#endif

 
}



test_collision()       
{
  unsigned int a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,s;
  int x_mom,x_mom2,y_mom,y_mom2,partsum,partsum2; 
  g=0;
  ng=0;
  for(a=0;a<2;a++)
    for(b=0;b<2;b++)
      for(c=0;c<2;c++)
	for(d=0;d<2;d++)
	  for(e=0;e<2;e++)
	    for(f=0;f<2;f++)
		{
		  s=0;
		  x_mom = 2*a - 2*d + b + f - e - c;
		  y_mom = b + c -  e - f;
		  partsum = a + b + c + d + e + f;
		  a << 31;
		  b << 31;
		  c << 31;
		  d << 31;
		  e << 31;
		  f << 31;
		  collision_6_right(a,b,c,d,e,f,g,&na,&nb,&nc,&nd,&ne,&nf,&ng,s);
		  na >> 31;
		  nb >> 31;
		  nc >> 31;
		  nd >> 31;
		  ne >> 31;
		  nf >> 31;
		  y_mom2 = nb + nc -  ne - nf;
		  partsum2 = na + nb + nc + nd + ne + nf;
		  x_mom2 =  2*na - 2*nd + nb + nf - ne - nc;
		  if(x_mom != x_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"x_mom");
		  if(y_mom != y_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"y_mom");
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"partsum");
		  collision_6_left(a,b,c,d,e,f,g,&na,&nb,&nc,&nd,&ne,&nf,&ng,s);
		  na >> 31;
		  nb >> 31;
		  nc >> 31;
		  nd >> 31;
		  ne >> 31;
		  nf >> 31;
		  x_mom2 =  2*na - 2*nd + nb + nf - ne - nc;
		  y_mom2 = nb + nc -  ne - nf;
		  partsum2 = na + nb + nc + nd + ne + nf;
		  if(x_mom != x_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left x_mom");
		  if(y_mom != y_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left y_mom");
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left partsum");
		  s=1;
		  partsum = a + b + c + d + e + f;
		  collision_6_right(a,b,c,d,e,f,g,&na,&nb,&nc,&nd,&ne,&nf,&ng,s);
		  na >> 31;
		  nb >> 31;
		  nc >> 31;
		  nd >> 31;
		  ne >> 31;
		  nf >> 31;
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"partsum s=1");
		  collision_6_left(a,b,c,d,e,f,g,&na,&nb,&nc,&nd,&ne,&nf,&ng,s);
		  na >> 31;
		  nb >> 31;
		  nc >> 31;
		  nd >> 31;
		  ne >> 31;
		  nf >> 31;
		  partsum2 = na + nb + nc + nd + ne + nf;
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left partsum s=1");
		}
 printf("Collisions_6 tested\n");
}

errorc(a, b, c, d, e, f, g, na, nb, nc, nd, ne, nf, ng, message)  unsigned int a; unsigned int b;unsigned int c;unsigned int d; unsigned int e; unsigned int f; unsigned int g; unsigned int na; unsigned int nb;unsigned int nc;unsigned int nd; unsigned int ne; unsigned int nf; unsigned int ng; char * message;
{
  fprintf(stderr,"error at %d %d %d %d %d %d %d for %s\n",a,b,c,d,e,f,g,message); 
}
