/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#include <stdio.h>
#include "declarations_7.h"

/* collision 7 parts */ 

void collision_7_left(unsigned int a, unsigned int b,unsigned int c,unsigned int d, unsigned int e, unsigned int f, unsigned int *g, unsigned int *na, unsigned int *nb,unsigned int *nc,unsigned int *nd, unsigned int *ne, unsigned int *nf, unsigned int s)
{
   unsigned int t1,t2,t3,t4,t5,t6,u1,u2,u3,g1,g2,g3,del,c0,c1,c2,c3,c4,c5,c6,e1,e2,e3,e4,e5,e6;

   t1 = a^b;
   t2 = b^c;
   t3 = c^d;
   t4 = d^e;
   t5 = e^f;
   t6 = f^a;
   

   e1 = t1 & t6 & ( *g^b );
   e2 = t2 & t1 & ( *g^c );
   e3 = t3 & t2 & ( *g^d );
   e4 = t4 & t3 & ( *g^e ); 
   e5 = t5 & t4 & ( *g^f ); 
   e6 = t6 & t5 & ( *g^a ); 

   u1 = t1 & t4;
   u2 = t2 & t5;
   u3 = t3 & t6;
   
/* g? means pair of opposing particles */ 

   g1 = u1 & ( b^d );
   g2 = u2 & ( c^e );
   g3 = u3 & ( d^f );

/* del means triple collision, with or without rest particles */ 
   
   del = u1 & u2 & u3;
	      
/* c0 is the condition for involving a rest particle */ 

   c0 = ~(del | ( g1 | (e1^(~e4))) & ( g2 | (e2 ^ (~e5))) & ( g3 | (e3^(~e6))));

   /* rotate left */
   /* If no c), as in 6 particle. Else, collide with rest part. */ 
	      
   c1 = del | ((~c0) & (g1 | g3 & ~g2)) | (c0 & (e6 | ( e1 & (~e3)) | e2 & ( ~e4) ));
   c2 = del | ((~c0) & (g2 | g1 & ~g3)) | (c0 & (e1 | ( e2 & (~e4)) | e3 & ( ~e5) ));
   c3 = del | ((~c0) & (g3 | g2 & ~g1)) | (c0 & (e2 | ( e3 & (~e5)) | e4 & ( ~e6) ));
   c4 = del | ((~c0) & (g1 | g3 & ~g2)) | (c0 & (e3 | ( e4 & (~e6)) | e5 & ( ~e1) ));
   c5 = del | ((~c0) & (g2 | g1 & ~g3)) | (c0 & (e4 | ( e5 & (~e1)) | e6 & ( ~e2) ));
   c6 = del | ((~c0) & (g3 | g2 & ~g1)) | (c0 & (e5 | ( e6 & (~e2)) | e1 & ( ~e3) ));

#ifdef SOLID_SITES
   *na =((~s) & (c1^a)) |( s & d);
   *nb =((~s) & (c2^b)) |( s & e);
   *nc =((~s) & (c3^c)) |( s & f);
   *nd =((~s) & (c4^d)) |( s & a);
   *ne =((~s) & (c5^e)) |( s & b);
   *nf =((~s) & (c6^f)) |( s & c);
   *g =((~s) & (c0^*g)) |( s & *g);
#else
   *na = c1^a;
   *nb = c2^b;
   *nc = c3^c;
   *nd = c4^d;
   *ne = c5^e;
   *nf = c6^f;
   *g = c0^*g; 
#endif
}

void collision_7_right(unsigned int a, unsigned int b,unsigned int c,unsigned int d, unsigned int e, unsigned int f, unsigned int *g, unsigned int *na, unsigned int *nb,unsigned int *nc,unsigned int *nd, unsigned int *ne, unsigned int *nf,unsigned int s)
{
   unsigned int t1,t2,t3,t4,t5,t6,u1,u2,u3,g1,g2,g3,del,c0,c1,c2,c3,c4,c5,c6,e1,e2,e3,e4,e5,e6;

   t1 = a^b;
   t2 = b^c;
   t3 = c^d;
   t4 = d^e;
   t5 = e^f;
   t6 = f^a;

   e1 = t1 & t6 & ( *g^b );
   e2 = t2 & t1 & ( *g^c );
   e3 = t3 & t2 & ( *g^d );
   e4 = t4 & t3 & ( *g^e ); 
   e5 = t5 & t4 & ( *g^f ); 
   e6 = t6 & t5 & ( *g^a ); 

   u1 = t1 & t4;
   u2 = t2 & t5;
   u3 = t3 & t6;
   
/* g means pair of opposing particles */ 

   g1 = u1 & ( b^d );
   g2 = u2 & ( c^e );
   g3 = u3 & ( d^f );

/* del means triple collision, with or without rest particles */ 
   
   del = u1 & u2 & u3;
	      
/* c0 is condition for involving a rest particle */ 

   c0 = ~(del | ( g1 | (e2^(~e5))) & ( g2 | (e3 ^ (~e6))) & ( g3 | (e1^(~e4))));

   /* rotate right */
   /* If no c), as in 6 particle. Else, collide with rest part. */ 
		 
   c1 = del | ((~c0) & (g3 | g1 & ~g2)) | (c0 & (e2 | ( e1 & (~e5)) | e6 & ( ~e4) ));
   c2 = del | ((~c0) & (g1 | g2 & ~g3)) | (c0 & (e3 | ( e2 & (~e6)) | e1 & ( ~e5) ));
   c3 = del | ((~c0) & (g2 | g3 & ~g1)) | (c0 & (e4 | ( e3 & (~e1)) | e2 & ( ~e6) ));
   c4 = del | ((~c0) & (g3 | g1 & ~g2)) | (c0 & (e5 | ( e4 & (~e2)) | e3 & ( ~e1) ));
   c5 = del | ((~c0) & (g1 | g2 & ~g3)) | (c0 & (e6 | ( e5 & (~e3)) | e4 & ( ~e2) ));
   c6 = del | ((~c0) & (g2 | g3 & ~g1)) | (c0 & (e1 | ( e6 & (~e4)) | e5 & ( ~e3) ));

#ifdef SOLID_SITES
   *na =((~s) & (c1^a)) |( s & d);
   *nb =((~s) & (c2^b)) |( s & e);
   *nc =((~s) & (c3^c)) |( s & f);
   *nd =((~s) & (c4^d)) |( s & a);
   *ne =((~s) & (c5^e)) |( s & b);
   *nf =((~s) & (c6^f)) |( s & c);
   *g =((~s) & (c0^*g)) |( s & *g);
#else
   *na = c1^a;
   *nb = c2^b;
   *nc = c3^c;
   *nd = c4^d;
   *ne = c5^e;
   *nf = c6^f;
   *g = c0^*g; 
#endif
}


void test_collision(void)       
{
  unsigned int a,b,c,d,e,f,g,tg,na,nb,nc,nd,ne,nf,ng,s;
  int x_mom,x_mom2,y_mom,y_mom2,partsum,partsum2; 
  for(a=0;a<2;a++)
    for(b=0;b<2;b++)
      for(c=0;c<2;c++)
	for(d=0;d<2;d++)
	  for(e=0;e<2;e++)
	    for(f=0;f<2;f++)
	      for(tg=0;tg<2;tg++)
		{
		  g = tg; 
		  s=0;
		  x_mom = 2*a - 2*d + b + f - e - c;
		  y_mom = b + c -  e - f;
		  partsum = a + b + c + d + e + f + g;
		  g <<= 31;
		  collision_7_right(a<<31,b<<31,c<<31,d<<31,e<<31,f<<31,&g,&na,&nb,&nc,&nd,&ne,&nf,s);
		  ng = g;
		  na >>= 31;
		  nb >>= 31;
		  nc >>= 31;
		  nd >>= 31;
		  ne >>= 31;
		  nf >>= 31;
		  ng >>= 31;
		  x_mom2 =  2*na - 2*nd + nb + nf - ne - nc;
		  y_mom2 = nb + nc -  ne - nf;
		  partsum2 = na + nb + nc + nd + ne + nf + ng;
		  if(x_mom != x_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"x_mom");
		  if(y_mom != y_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"y_mom");
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"partsum");
		  g = tg; g <<= 31;
		  collision_7_left(a<<31,b<<31,c<<31,d<<31,e<<31,f<<31,&g,&na,&nb,&nc,&nd,&ne,&nf,s);
		  ng = g;
		  na >>= 31;
		  nb >>= 31;
		  nc >>= 31;
		  nd >>= 31;
		  ne >>= 31;
		  nf >>= 31;
		  ng >>= 31;
		  x_mom2 =  2*na - 2*nd + nb + nf - ne - nc;
		  x_mom2 =  2*na - 2*nd + nb + nf - ne - nc;
		  y_mom2 = nb + nc -  ne - nf;
		  partsum2 = na + nb + nc + nd + ne + nf + ng;
		  if(x_mom != x_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left x_mom");
		  if(y_mom != y_mom2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left y_mom");
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left partsum");
		  s=1;
		  g = tg; g <<= 31;
		  collision_7_right(a<<31,b<<31,c<<31,d<<31,e<<31,f<<31,&g,&na,&nb,&nc,&nd,&ne,&nf,s);
		  ng = g;
		  na >>= 31;
		  nb >>= 31;
		  nc >>= 31;
		  nd >>= 31;
		  ne >>= 31;
		  nf >>= 31;
		  ng >>= 31;
		  partsum2 = na + nb + nc + nd + ne + nf + ng;
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"partsum s=1");
		  g = tg;  g <<= 31;
		  collision_7_left(a<<31,b<<31,c<<31,d<<31,e<<31,f<<31,&g,&na,&nb,&nc,&nd,&ne,&nf,s);
		  ng = g;
		  na >>= 31;
		  nb >>= 31;
		  nc >>= 31;
		  nd >>= 31;
		  ne >>= 31;
		  nf >>= 31;
		  ng >>= 31;
		  partsum2 = na + nb + nc + nd + ne + nf + ng;
		  if(partsum != partsum2) errorc(a,b,c,d,e,f,g,na,nb,nc,nd,ne,nf,ng,"left partsum s=1");
		}
 printf("Collisions 7 tested\n");
}

