/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#line 1 "interact.c"
/* 
  $Author: zaleski $
  $Revision: 1.2 $
*/

#include "macros_gen.h"
   
/* 

From theory (e.g. RMP equation ...)

n_i^'(X) = n_i(X) + gamma_i - gamma_-i 

where 

 gamma_i  = ~n_i(X) & n_{-i}(X) &  n_i(X+ rC_i) & ~n_{-i}(X + rC_i) 
 gamma_-i =  n_i(X) & ~n_{-i}(X) & ~n_i(X- rC_i) &  n_{-i}(X - rC_i) 


n_i^' = n_i XOR (gamma_i + gamma_-i) 

*/



/* 
The macro below is helpful for writing more concisely 
the interaction step on every boundary
and corner. As input it takes the coordinates 
of the site (ii, jj) and those (ii_dn, jj_rdn), etc... 
of the neighboring words useful for the extraction of 
sites at distance r on the hexagonal axes.
*/ 
 
#define INTERACT(ii,ii_dn,ii_up,jj,jj_rdn,jj_rup) \
  zword  = (ii) + max_x_word_coord * (jj); \
  zword_up = (ii) + max_x_word_coord * (jj_rup); \
  zword_dn  = (ii) + max_x_word_coord * (jj_rdn);\
  zword_upl = (ii_dn) + max_x_word_coord * (jj_rup); \
  zword_dnr  = (ii_up) + max_x_word_coord * (jj_rdn);\
  zword_next_right = (ii_up) +  max_x_word_coord * (jj);\
  zword_next_left = (ii_dn) +  max_x_word_coord * (jj); \
  g1 =  ~a[zword] & d[zword] &  \
            ((a[zword_next_right] >> wrdlnminr) | (a[zword] << r)) & \
	   ~((d[zword_next_right] >> wrdlnminr) | (d[zword] << r));  \
  g1 |=  ~d[zword] & a[zword] & \
           ~((a[zword_next_left] << wrdlnminr) | (a[zword] >> r)) & \
	    ((d[zword_next_left] << wrdlnminr) | (d[zword] >> r));  \
  na[zword] = a[zword]^g1; \
  nd[zword] = d[zword]^g1; \
  g1 = ~b[zword] & e[zword] & b[zword_up]  & ~e[zword_up];  \
  g1 |= ~e[zword] & b[zword] & ~b[zword_dn] &  e[zword_dn];  \
  nb[zword] = b[zword]^g1; \
  ne[zword] = e[zword]^g1; \
  g1 = ~c[zword] & f[zword] &  \
            ((c[zword_upl] << wrdlnminr) | (c[zword_up] >> r)) & \
	   ~((f[zword_upl] << wrdlnminr) | (f[zword_up] >> r)); \
  g1 |= ~f[zword] & c[zword] &  \
          ~((c[zword_dnr] >> wrdlnminr) | (c[zword_dn] <<  r)) & \
	   ((f[zword_dnr] >> wrdlnminr) | (f[zword_dn] <<  r)); \
  nc[zword] = c[zword]^g1; \
  nf[zword] = f[zword]^g1;


 
#define INTERACTB(ii,ii_dn,ii_up,jj,jj_rdn,jj_rup) \
  zword  = (ii) + max_x_word_coord * (jj); \
  zword_up = (ii) + max_x_word_coord * (jj_rup); \
  zword_dn  = (ii) + max_x_word_coord * (jj_rdn);\
  zword_upl = (ii_dn) + max_x_word_coord * (jj_rup); \
  zword_dnr  = (ii_up) + max_x_word_coord * (jj_rdn);\
  zword_next_right = (ii_up) +  max_x_word_coord * (jj);\
  zword_next_left = (ii_dn) +  max_x_word_coord * (jj); \
  g1 = ~b[zword] & e[zword] & b[zword_up]  & ~e[zword_up];  \
  g1 |= ~e[zword] & b[zword] & ~b[zword_dn] &  e[zword_dn];  \
  nb[zword] = b[zword]^g1; \
  ne[zword] = e[zword]^g1; \
  na[zword] = a[zword]; \
  nc[zword] = c[zword]; \
  nd[zword] = d[zword]; \
  nf[zword] = f[zword]; 




interact(a, b, c, d, e, f, na, nb, nc, nd, ne, nf, r, nx, ny)  unsigned int *a; unsigned int *b;unsigned int *c;unsigned int *d; unsigned int *e; unsigned int *f; unsigned int *na; unsigned int *nb;unsigned int *nc;unsigned int *nd; unsigned int *ne; unsigned int *nf; int r; int nx; int ny;
{
  unsigned int g1;
  int i,j,j_rdn,j_rup;
  int zword, iword, max_x_word_coord;
  int zword_up,  zword_dn, zword_next_right, zword_next_left;
  int zword_upl, zword_dnr, wrdlnminr;

  wrdlnminr = word_length - r; 
  max_x_word_coord=nx/word_length;
	  

/* INTERACTION */

      for(j=0;j < ny; j++)
	{
	  j_rdn = WRAP_AROUND(j-r,ny);
	  j_rup = WRAP_AROUND(j+r,ny);
	  for(iword=1;iword < max_x_word_coord-1; iword++)
	    {
	      INTERACT(iword,iword-1,iword+1,j,j_rdn,j_rup)
	    }
	      /* Treat border lines with wrap around condition  */
	    INTERACT(0,max_x_word_coord-1,1,j,j_rdn,j_rup)
            INTERACT(max_x_word_coord-1,max_x_word_coord-2,0,j,j_rdn,j_rup)
	  }
}


