/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#line 1 "interact32.c"
/* 
  $Author$
  $Revision$
*/

#include "macros_gen.h"
   
/* 

From theory (e.g. RMP equation ...)

n_i^'(X) = n_i(X) + gamma_i - gamma_-i 

where 

 gamma_i  =  ~n_i(X) & n_{-i}(X) &  n_i(X+ rE_i) & ~n_{-i}(X + rE_i) 
 gamma_-i =  n_i(X) & ~n_{-i}(X) & ~n_i(X- rE_i) &  n_{-i}(X - rE_i) 


n_i^' = n_i XOR (gamma_i + gamma_-i) 

and r E_i is located with respect to the origin as on the figure below.

Here we interact on all sites at distances between 2 and 3 from 
the origin 

                                   /j direction 
                                  /
     .   .   .   .   .   .   .   .   .

       .   .   .   .   .   .   .   .   .

     .   .   .   .   .   .   .   .   .

       .   .   x   B   A  x   .   .   .
                 d2u3 d1u3
     .   .   A  x   .   x   B   .   . 
           d3u2            u1u2
       .   B   .   .   .   .   A  .   .
                             u2u1
     .   x   x   .   o   .   x   x   .---------------> i direction

       .   A   .   .   .   .   B   .   .
                             u3d1
     .   .   B   x   .   x   A   .   .
                           u3d2
       .   .   x   A   B   x   .   .   .
                      u2d3
     .   .   .   .   .   .   .   .   .

       .   .   .   .   .   .   .   .   .


There are 2 sets of 6 sites labelled A and B 

The A sites interact like this:

         .     .     .     . <===A--->


        <---O===> .     .     .     .

where  <--- is the velocity before interaction and  ===> after.


The indices on the picture (u2u1 etc..) are examples of the coordinates
used in the macros below. These macros are helpful for writing more 
concisely the interaction step on every boundary. 
As input they take the coordinates 
of the wordf (ii, jj) and those (ii_dn, jj_1dn, jj_2dn), etc... 
of the neighboring words useful for the extraction of 
sites at distance r on the hexagonal axes.

*/ 
 

#define INTERACT_32A(ii,ii_dn,ii_up,jj,jj_1dn,jj_2dn,jj_3dn,jj_1up,jj_2up,jj_3up) \
  zword  = (ii) + max_x_word_coord * (jj); \
  zword_u1 = (ii) + max_x_word_coord * (jj_1up); \
  zword_d1  = (ii) + max_x_word_coord * (jj_1dn);\
  zword_u2 = (ii) +  max_x_word_coord * (jj_2up);\
  zword_d2 = (ii) +  max_x_word_coord * (jj_2dn); \
  zword_u3 = (ii) +  max_x_word_coord * (jj_3up);\
  zword_d3 = (ii) +  max_x_word_coord * (jj_3dn); \
  zword_u2u1 = (ii_up) +  max_x_word_coord * (jj_1up);\
  zword_d2d1 = (ii_dn) +  max_x_word_coord * (jj_1dn); \
  zword_d1u3 = (ii_dn) +  max_x_word_coord * (jj_3up);\
  zword_u1d3 = (ii_up) +  max_x_word_coord * (jj_3dn); \
  zword_d3u2 = (ii_dn) +  max_x_word_coord * (jj_2up);\
  zword_u3d2 = (ii_up) +  max_x_word_coord * (jj_2dn); \
  g1 =  ~a[zword] & d[zword] &  \
            ((a[zword_u2u1] >> wrdlnmin2) | (a[zword_u1] << 2)) & \
	   ~((d[zword_u2u1] >> wrdlnmin2) | (d[zword_u1] << 2));  \
  g1 |=  ~d[zword] & a[zword] & \
           ~((a[zword_d2d1] << wrdlnmin2) | (a[zword_d1] >> 2)) & \
	    ((d[zword_d2d1] << wrdlnmin2) | (d[zword_d1] >> 2));  \
  na[zword] = a[zword]^g1; \
  nd[zword] = d[zword]^g1; \
  g1 = ~b[zword] & e[zword] & \
          ((b[zword_d1u3] << wrdlnmin1 ) | (b[zword_u3] >> 1 )) &  \
         ~((e[zword_d1u3] << wrdlnmin1 ) | (e[zword_u3] >> 1 ));  \
  g1 |= ~e[zword] & b[zword] & \
          ~((b[zword_u1d3] >> wrdlnmin1) | (b[zword_d3] << 1) ) &  \
           ((e[zword_u1d3] >> wrdlnmin1) | (e[zword_d3] << 1) );  \
  nb[zword] = b[zword]^g1; \
  ne[zword] = e[zword]^g1; \
  g1 = ~c[zword] & f[zword] &  \
            ((c[zword_d3u2] << wrdlnmin3) | (c[zword_u2] >> 3)) & \
	   ~((f[zword_d3u2] << wrdlnmin3) | (f[zword_u2] >> 3)); \
  g1 |= ~f[zword] & c[zword] &  \
          ~((c[zword_u3d2] >> wrdlnmin3) | (c[zword_d2] << 3)) & \
	   ((f[zword_u3d2] >> wrdlnmin3) | (f[zword_d2] << 3)); \
  nc[zword] = c[zword]^g1; \
  nf[zword] = f[zword]^g1;

#define INTERACT_32B(ii,ii_dn,ii_up,jj,jj_1dn,jj_2dn,jj_3dn,jj_1up,jj_2up,jj_3up) \
  zword  = (ii) + max_x_word_coord * (jj); \
  zword_u1 = (ii) + max_x_word_coord * (jj_1up); \
  zword_d1  = (ii) + max_x_word_coord * (jj_1dn);\
  zword_u2 = (ii) +  max_x_word_coord * (jj_2up);\
  zword_d2 = (ii) +  max_x_word_coord * (jj_2dn); \
  zword_u3 = (ii) +  max_x_word_coord * (jj_3up);\
  zword_d3 = (ii) +  max_x_word_coord * (jj_3dn); \
  zword_u3d1 = (ii_up) +  max_x_word_coord * (jj_1dn);\
  zword_d3u1 = (ii_dn) +  max_x_word_coord * (jj_1up); \
  zword_u1u2 = (ii_up) +  max_x_word_coord * (jj_2up);\
  zword_d1d2 = (ii_dn) +  max_x_word_coord * (jj_2dn); \
  zword_d2u3 = (ii_dn) +  max_x_word_coord * (jj_3up);\
  zword_u2d3 = (ii_up) +  max_x_word_coord * (jj_3dn); \
  g1 =  ~a[zword] & d[zword] &  \
            ((a[zword_u3d1] >> wrdlnmin3) | (a[zword_d1] << 3)) & \
	   ~((d[zword_u3d1] >> wrdlnmin3) | (d[zword_d1] << 3));  \
  g1 |=  ~d[zword] & a[zword] & \
           ~((a[zword_d3u1] << wrdlnmin3) | (a[zword_u1] >> 3)) & \
	    ((d[zword_d3u1] << wrdlnmin3) | (d[zword_u1] >> 3));  \
  na[zword] = a[zword]^g1; \
  nd[zword] = d[zword]^g1; \
  g1 = ~b[zword] & e[zword] & \
          ((b[zword_u1u2] >> wrdlnmin1 ) | (b[zword_u2] << 1 )) &  \
         ~((e[zword_u1u2] >> wrdlnmin1 ) | (e[zword_u2] << 1 ));  \
  g1 |= ~e[zword] & b[zword] & \
          ~((b[zword_d1d2] << wrdlnmin1) | (b[zword_d2] >> 1) ) &  \
           ((e[zword_d1d2] << wrdlnmin1) | (e[zword_d2] >> 1) );  \
  nb[zword] = b[zword]^g1; \
  ne[zword] = e[zword]^g1; \
  g1 = ~c[zword] & f[zword] &  \
            ((c[zword_d2u3] << wrdlnmin2) | (c[zword_u3] >> 2)) & \
	   ~((f[zword_d2u3] << wrdlnmin2) | (f[zword_u3] >> 2)); \
  g1 |= ~f[zword] & c[zword] &  \
          ~((c[zword_u2d3] >> wrdlnmin2) | (c[zword_d3] << 2)) & \
	   ((f[zword_u2d3] >> wrdlnmin2) | (f[zword_d3] << 2)); \
  nc[zword] = c[zword]^g1; \
  nf[zword] = f[zword]^g1;


 

interact32a(a, b, c, d, e, f, na, nb, nc, nd, ne, nf, nx, ny)  unsigned int *a; unsigned int *b; unsigned int *c; unsigned int *d; unsigned int *e; unsigned int *f; unsigned int *na; unsigned int *nb;unsigned int *nc;unsigned int *nd; unsigned int *ne; unsigned int *nf; int nx; int ny;
{
  unsigned int g1;
  int i,j;
  int zword, iword, max_x_word_coord;
  int zword_u1,zword_d1, zword_u2u1, zword_d2d1, zword_d1u3, zword_u1d3;
  int zword_d3u2, zword_u3d2, zword_u2, zword_d2, zword_u3, zword_d3;
  int  wrdlnmin1,  wrdlnmin2,  wrdlnmin3;
  int j_1dn, j_2dn, j_3dn, j_1up, j_2up, j_3up; 

  wrdlnmin1 = word_length_minus_1; 
  wrdlnmin2 = word_length - 2; 
  wrdlnmin3 = word_length - 3; 
  max_x_word_coord=nx/word_length;
	  

/* INTERACTION */

      for(j=0;j < ny; j++)
	{
	  j_1up = WRAP_AROUND(j+1,ny);
	  j_1dn = WRAP_AROUND(j-1,ny);
	  j_2up = WRAP_AROUND(j+2,ny);
	  j_2dn = WRAP_AROUND(j-2,ny);
	  j_3up = WRAP_AROUND(j+3,ny);
	  j_3dn = WRAP_AROUND(j-3,ny);
	  for(iword=1;iword < max_x_word_coord-1; iword++)
	    {
	      INTERACT_32A(iword,iword-1,iword+1,j,j_1dn,j_2dn,j_3dn,j_1up,j_2up,j_3up)
	    }
	      /* Treat border lines with wrap around condition  */
	    INTERACT_32A(0,max_x_word_coord-1,1,j,j_1dn,j_2dn,j_3dn,j_1up,j_2up,j_3up)
            INTERACT_32A(max_x_word_coord-1,max_x_word_coord-2,0,j,j_1dn,j_2dn,j_3dn,j_1up,j_2up,j_3up)
	  }
}

interact32b(a, b, c, d, e, f, na, nb, nc, nd, ne, nf, nx, ny)  unsigned int *a; unsigned int *b; unsigned int *c; unsigned int *d; unsigned int *e; unsigned int *f; unsigned int *na; unsigned int *nb;unsigned int *nc;unsigned int *nd; unsigned int *ne; unsigned int *nf; int nx; int ny;
{
  unsigned int g1;
  int i,j, word_length;
  int zword, iword, max_x_word_coord;
  int zword_u1,zword_d1, zword_u3d1, zword_d3u1, zword_d1d2, zword_u1u2;
  int zword_d2u3, zword_u2d3, zword_u2, zword_d2, zword_u3, zword_d3;
  int  wrdlnmin1,  wrdlnmin2,  wrdlnmin3;
  int j_1dn, j_2dn, j_3dn, j_1up, j_2up, j_3up; 

  word_length  = sizeof(int)*8;
  wrdlnmin1 = word_length_minus_1; 
  wrdlnmin2 = word_length - 2; 
  wrdlnmin3 = word_length - 3; 
  max_x_word_coord=nx/word_length;
	  

/* INTERACTION */

      for(j=0;j < ny; j++)
	{
	  j_1up = WRAP_AROUND(j+1,ny);
	  j_1dn = WRAP_AROUND(j-1,ny);
	  j_2up = WRAP_AROUND(j+2,ny);
	  j_2dn = WRAP_AROUND(j-2,ny);
	  j_3up = WRAP_AROUND(j+3,ny);
	  j_3dn = WRAP_AROUND(j-3,ny);
	  for(iword=1;iword < max_x_word_coord-1; iword++)
	    {
	      INTERACT_32B(iword,iword-1,iword+1,j,j_1dn,j_2dn,j_3dn,j_1up,j_2up,j_3up)
	    }
	      /* Treat border lines with wrap around condition  */
            INTERACT_32B(max_x_word_coord-1,max_x_word_coord-2,0,j,j_1dn,j_2dn,j_3dn,j_1up,j_2up,j_3up)
	    INTERACT_32B(0,max_x_word_coord-1,1,j,j_1dn,j_2dn,j_3dn,j_1up,j_2up,j_3up)
	  }
}


