/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#line 1 "print_fhp7.c"
/* 
  $Author: zaleski $
  $Revision$ 
*/
#include "macros_gen.h"
#include "macros_7.h"
#include <stdio.h> 

/* INITIAL CONDITIONS */

print_fhp7(latt, lat_out, a, b, c, d, e, f, g, s, nx, ny, outfd)  unsigned int *latt; unsigned char *lat_out; unsigned int *a; unsigned int *b; unsigned int *c; unsigned int *d; unsigned int *e; unsigned int *f; unsigned int *g; unsigned int *s; int nx; int ny; FILE *outfd;
{
  int i,j,x,y,nz,iword, zlatt;
  int bit_order;
  int bit_from_left;
  int zword, word_length, max_x_word_coord; 
  
  nz = nx*ny;
  word_length=8*sizeof(int);
  max_x_word_coord=nx/word_length;
  
  
  for(j=0;j < ny; j++)
    {
      for(i=0;i < nx; i++)
	{
	  zlatt = i + nx*j;
	  iword = i / word_length;
	  zword = iword + max_x_word_coord * j; 
	  latt[zlatt] = 0; 

	  /* Bit counted from left of word */

	  bit_from_left = i - iword*word_length; 

	  /* BIt order in word (counted from right) */

	  bit_order = word_length_minus_1  - bit_from_left;
	  
	  /* Mask multisite word, 
	     push bit to the right of multisite word a,b,c ,..., 
	     then left to correct position on single site word latt[z] 
	     */
	  latt[zlatt]  =  (a[zword] >> bit_order) & 1 ; 
	  latt[zlatt] |= ((b[zword] >> bit_order) & 1 )  << 1 ;
	  latt[zlatt] |= ((c[zword] >> bit_order) & 1 )  << 2 ;
	  latt[zlatt] |= ((d[zword] >> bit_order) & 1 )  << 3 ;
	  latt[zlatt] |= ((e[zword] >> bit_order) & 1 )  << 4 ;
	  latt[zlatt] |= ((f[zword] >> bit_order) & 1 )  << 5 ;
	  latt[zlatt] |= ((g[zword] >> bit_order) & 1 )  << 6 ;
	  latt[zlatt] |= ((s[zword] >> bit_order) & 1 )  << 7 ;
	}
    }
  
  /* Transfer to cartesian grid */ 
  
  for(i=0;i<nx;i++)
    for(j=0;j<ny;j++)
      {
	x = X_HEX_TO_CART(i,j);
	y = Y_HEX_TO_CART(i,j); 
	lat_out[x + nx*y] = latt[i + nx*j];
      }
  fwrite((char *) lat_out, sizeof(char), nz*sizeof(char),outfd);
} 
