#include <stdio.h>
#include <math.h>
#include "boltzmann.h"

#define MINIMALNORM 	0.01		
#define RAD             180/PI


static unsigned char coltab[12][6]={{0,1,5,2,4,3},
				    {1,0,2,5,3,4},
				    {1,2,0,3,5,4},
				    {2,1,3,0,4,5},
				    {2,3,1,4,0,5},
				    {3,2,4,1,5,0},
				    {3,4,2,5,1,0},
				    {4,3,5,2,0,1},
				    {4,5,3,0,2,1},
				    {5,4,0,3,1,2},
				    {5,0,4,1,3,2},
				    {0,5,1,4,2,3}}; 

				    
/*----------------------------------------------------------------------*/

int TableEntry(double x, double y)
{
  int index;
  
  index=(int) (atan2(y,x)*6/PI);
  if(index<0)
    index = 12+index;
  return index;
}

/*----------------------------------------------------------------------*/


double Mass(int ix, int iy, double *lat)
{
  int i;
  double mass;
/* need to add the color of the walls in initrock routine.
   right now we assume the walls blue (nonwedding for red) */

  for(mass=0.0,i=0;i<6;i++) mass += lat[(ix+iy*NX)*6+i];

  return mass;
}


/*----------------------------------------------------------------------*/
void ColorGradient(int ix, int iy, double* red, double *blue, double f[])
{

  int i, alpha;
  int xp1,yp1,xm1,ym1;
  double r[6], b[6];
  

  xp1=(ix+1+NX)%NX;
  yp1=(iy+1+NY)%NY;
  xm1=(ix-1+NX)%NX;
  ym1=(iy-1+NY)%NY;

  
  r[0]=Mass(xp1, iy, red);
  r[3]=Mass(xm1, iy, red);
  b[0]=Mass(xp1, iy, blue);
  b[3]=Mass(xm1, iy, blue);
  if(iy&1) /* odd rows */
    {
  
      r[1]=Mass(xp1,yp1,red);
      b[1]=Mass(xp1,yp1,blue);
      r[2]=Mass(ix,yp1,red);
      b[2]=Mass(ix,yp1,blue);
      r[4]=Mass(ix,ym1,red);
      b[4]=Mass(ix,ym1,blue);
      r[5]=Mass(xp1,ym1,red);
      b[5]=Mass(xp1,ym1,blue);
    }
  else /* iy= 0 and even */
    {
      r[1]=Mass(ix,yp1,red);
      b[1]=Mass(ix,yp1,blue);
      r[2]=Mass(xm1,yp1,red);
      b[2]=Mass(xm1,yp1,blue);
      r[4]=Mass(xm1,ym1,red);
      b[4]=Mass(xm1,ym1,blue);
      r[5]=Mass(ix,ym1,red);
      b[5]=Mass(ix,ym1,blue);
    }
  
  for(i=0,f[0]=0.0,f[1]=0.0;i<6;i++)
  {
    for(alpha=0;alpha<2;alpha++)
      f[alpha] += C(i,alpha)*(r[i]-b[i]);
  }

}

/*----------------------------------------------------------------------*/

void ReDistRed(int ix,
	       int iy, 
	       char *solid, 
	       double *red, 
	       double *blue, 
	       double A)
{
  int i,k, alpha, entry;
  double q, temp, r, dr,f[2], L2f, N[6], dN, angle;

  ColorGradient(ix,iy,red, blue,f);
  L2f = sqrt(f[0]*f[0]+f[1]*f[1]);

  if(L2f < MINIMALNORM)   /* some red is needed */
    return;
  else                         
    {
/*add surface tension perturbation */
      angle = atan2(f[1],f[0]);
      if(angle < 0.0)
	  angle += 2.0*PI;
      entry=(int) floor(angle*6.0/PI) % 12;  
      for(i=0,q=0.0,r=0.0;i<6;i++)
	{
	  dN=A*L2f*cos(2.0*(angle-q*PI/3.0));
	  N[i]=RED(ix,iy,i)+BLUE(ix,iy,i)+dN;
	  r += RED(ix,iy,i);
	  q += 1.0;
	}
/* maximize color gradient 
   through redistributing of total red, r */
      for(i=0;i<6;i++)
	{
	  k=coltab[entry][i];
	  dr=MIN(r,N[k]);  /* no more then there is, mass conv*/
	  dr=MAX(0.0,dr);  /* and always something */
	  RED(ix,iy,k)=dr;
	  BLUE(ix,iy,k)=N[k]-dr;
	  r -= dr;
	}
    }
}

/*----------------------------------------------------------------------*/
void TwoPhase(char *solid,double *red, double *blue,double A)
{
  int i, j, ix, iy;
  
  for(iy=0;iy<NY;iy++)
    {
    for(ix=0;ix<NX;ix++)
      ReDistRed(ix, iy, solid, red, blue, A);
    }

}

#ifdef TEM


void main()
{
  double x, y;
  int index;

  for(;;)
  {
    fprintf(stderr,"->x: ");
    scanf("%lf",&x);
    fprintf(stderr,"->y: ");
    scanf("%lf",&y);
    fprintf(stderr,
	    "angle x,y = atan(y/x) in rad: %f\n",RAD*atan2(y,x));
    index=TableEntry(x,y);
    fprintf(stderr,"index = %d\n",index);
    
  }
}





#endif









