/*************************************************
 * implementation of Andrew Gunstensen and Rothman
 * Boltzmann method for immiscible flow.
 * **********************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/*-----------------------------------------------*
 * globals
 *-----------------------------------------------*/
double *red, *blue;
char  *solid;
int NX, NY, FRAMES;
double omega[36], c[12],q[24], *temp;
double redprob, density,  fluct, LAMBDA, TAU;
long seed;

/*-----------------------------------------------*
 * macros
 *-----------------------------------------------*/
#define BOOL int
#define TRUE  1
#define FALSE 0
#define MIN(a,b) (((a)>(b)) ? (b):(a))
#define MAX(a,b) (((a)<(b)) ? (b):(a))

#ifndef PI
#define PI 3.141592654
#endif


#define RED(ix,iy,i)      red[((ix)+(iy)*NX)*6+(i)]
#define BLUE(ix,iy,i)     blue[((ix)+(iy)*NX)*6+(i)]
#define NEWLAT(ix,iy,i)   newlat[((ix)+(iy)*NX)*6+(i)]
#define SOLID(ix,iy)      solid[(ix)+(iy)*NX]
#define OMEGA(i,j)        omega[6*(i)+(j)]
#define C(i,alpha)        c[(i)*2+(alpha)]
#define Q(i,alpha,beta)   q[((i)*2+(alpha))*2+(beta)]
#define DELTA(i,j)        (((i)==(j)) ? 1 : 0)

/*-----------------------------------------------*
 * definitions
 *-----------------------------------------------*/


/*-----------------------------------------------*
 * prototypes
 *-----------------------------------------------*/
double ran1(long *idum);

void Stream(double *lat);
void SetUniverse(int nx, int ny);
void CleanUniverse();
void Model();
void TestCons(void);
void InitLat(char init_type, 
	     double redprob,
	     double density, 
	     double fluct,
	     double vx, double vy);
double ChanVisc(double gx);
void AddGravity(double *lat,double gx, double gy);
void ForceEqual(double gx, double gy);
void CleanUniverse();

void InitRock(char* s);
void SetSurface(double* pore, char *s);

int TableEntry(double x, double y);
double Mass(int ix, int iy, double *lat);
void ColorGradient(int ix, int iy, double* red, double *blue, double f[]);
void ReDistRed(int ix,
	       int iy, 
	       char *solid, 
	       double *red, 
	       double *blue, 
	       double A);

void TwoPhase(char *solid,double *red, double *blue,double A);
void MechSurfTension(double A);
double LaplaceLaw(int xc, int yc, double R);







