#ifndef BOLTZMANN_H
#define BOLTZMANN_H
#include "boltzmann.h"
#endif

#ifndef PI
#define PI 3.1415927
#endif

/* comments: need to get of that ranodm stuff! */


void InitLat(char inittype, 
	     double redprob,
	     double density, 
	     double fluct,
	     double vx, double vy )
{
/* ---------------------------------------* 
 * initialize lattice(perhaps implement fluctuation?)
 * initype: 'n' -> normal, specify rho, vx, vy
 * initype: 't' -> test for viscosity
 * ---------------------------------------*/
  int j, ix, iy;
  double k, r2;

/*
  options: u for uniform distribution
           h red/blue half spaces
	   r rayleigh taylor instab., below and less dense
	   v relaxation of a red wave. computing of 
	     the viscosity

*/
           
  for(iy = 0; iy < NY; iy++)
    for(ix = 0; ix < NX; ix++)
      {
	if(0 == SOLID(ix,iy))  /* only void space!*/
	  {
/* switch to options */
	    switch(inittype)
	      {
		  case 'b':    /* bubble */
		    {
		      double cx, cy, rx, ry;
		      double radius = (double)  NX/4.0;

		      if(iy&1)
			{
			  cx=(double) NX/2;
			  cy=(double) NY/2;
			  ry=(double) iy;
			  rx=(double) ix;
			  r2 =  3.0/4.0*(ry-cy)*(ry-cy)+
			    (rx-cx)*(rx-cx);
			}
		      else
			{
			  cx=(double) NX/2+0.5;
			  cy=(double) NY/2;
			  rx=(double) ix;
			  ry=(double) iy;
			  r2 =  3.0/4.0*(ry-cy)*(ry-cy)+
			  (rx-cx)*(rx-cx);
			}
		      for(j=0; j<6; j++)
			{
			  if(r2 < radius*radius)
			    BLUE(ix,iy,j)= (double) density/6.0;
			  else
			    RED(ix,iy,j) = (double) density/6.0;
			}
		    }
		  break;
	      case 'u':
		for(j=0; j<6; j++)
		  {
		    RED(ix,iy,j)= (double) 1.0+2*
		      (C(j,0)*vx+C(j,1)*vy);
		    RED(ix,iy,j) *= (1.0-fluct*ran1(&seed))* 
		      density/6;
		    BLUE(ix,iy,j) = (double) 1.0+
		      2*(C(j,0)*vx+C(j,1)*vy);
		    BLUE(ix,iy,j) *= (1.0-redprob)*
		      (1.0-fluct*ran1(&seed))*density/6;
		  }
		break;
	      case 'h':           /* upper half red */
		for(j=0; j<6; j++)
		  {
		    if(iy < NY/2)
		      RED(ix,iy,j)= (1.0-fluct*ran1(&seed))* 
			density/6;	
		    else
		      BLUE(ix,iy,j)=(1.0-fluct*ran1(&seed))* 
			density/6;
		  }
		break;
	      case 'r':
		{
		  int amplitude, d;
		  double wavenr;
		  
		  amplitude = 1.0;
		  wavenr = 4.0;

		  wavenr = (double) wavenr*2.0*PI/NX;
		  /*		  d = (int)  floor(amplitude* sin(wavenr*ix)); */
		  d = floor(amplitude* ran1(&seed));
	          d += NY/2;
		  for(j=0; j<6; j++)
		  {
		    if (iy<d)
		      RED(ix,iy,j)=density/6.0;
		    else
		      BLUE(ix,iy,j)=density/6.0;
		  }
		}
		break;
	      case 'v':
/* set velocity to (u(y),0), 
   where u(y) = umax*e^(k^2*nu*t)*sin(ky) */
/*		scanf(" %f",&k); */
		k=3.;
		for(j=0;j<6;j++)
		  {		    
		    RED(ix,iy,j)= density/6;
		    RED(ix,iy,j) += density/6*
		      C(j,0)*vx*sin(k*2.0*PI*iy/NY); 
		  }
		break;
	      }
	  }
      }  
}	    


double ChanVisc(double gx)
{
 int ix, iy;
 int i;
 double d2,size,rho,ux, mean_of_ux;

 mean_of_ux = size=0.0;
 for(ix=0;ix<NX;ix++)
   for(iy=0;iy<NY;iy++)
     {
       if(SOLID(ix,iy) == 0)
	  {
	    ux=rho=0.0;
	    for(i=0;i<6;i++)
	      {
		ux += C(i,0)*RED(ix,iy,i);
		rho += RED(ix,iy,i);
	      }
	    mean_of_ux += ux/rho;
	    size += 1.0;
	  }
	    
     }
 d2 = (float) (NY-1.5)*(NY-1.5)*3.0/4.0;
 mean_of_ux /= size;
 fprintf(stderr,"mean u %f measured: %f predicted: %f\n",
	 mean_of_ux,gx*d2/12.0/mean_of_ux,
	 (-1/4.)*(1/LAMBDA+.5));
}





double Viscosity()
{
/*--------------------------------------------------------------*
 * compute the area of velocity distribution:
 * |u(y,t) -1/2|=1/2*e^(-Dk^2t)*|sin(k(y)| 
 * area computed by simple Riemann summation:
 * A(t) = Sum(|u(x,t) - 1/2*vmax|)
 *--------------------------------------------------------------*/
 int ix, iy;
 int i;
 double rho,u;
 double A;

 A=0.0;
 for(ix=0;ix<NX;ix++)
   for(iy=0;iy<NY;iy++)
     {
       u=rho=0;
       for(i=0;i<6;i++)
	 {
	   rho += RED(ix,iy,i);
	   u += RED(ix,iy,i)*C(i,0);
	 }
       u /= rho;
       A += fabs(u);
     }  
 A /= (double) (NX); 
 return A;
}


#ifdef INITLAT	    
void main()
{
 int i,j,k;
 double A0,wavenr;
 
 k=5;
 LAMBDA=-.5;
 SetUniverse(10,100);
 fprintf(stderr,"predicted viscosity: %f\n", 
	 (-1/4.)*(1/LAMBDA+.5));
 InitLat('t',.5,.02,.0);


 TestCons(); 
 A0=Viscosity();

 wavenr=2*PI/NY * k;
   for(j=0;j<50;j++)
     {
       for(i=0;i<10;i++ )
	 {
	   Stream(RED); 
	   Collision();
	 }
       fprintf(stderr,"%d %f %f \n",(j+1)*(i),wavenr*wavenr*(i)*(j+1),
	     log(A0/Viscosity()));
/*       TestCons(); */
     }

 
}
#endif
    













