#include "boltzmann.h"

/*---------------------------------------------------------*
 * computation of mechanic surface tension across a flat
 * and horizontal interface. this routine evaluates 
 * expression (10.14) from R.Z. page 120
 *---------------------------------------------------------*/


void MechSurfTension(double A)
{
  int ix, iy, i;
  double cx2[6]={1.0,0.25,0.25,1.0,0.25,0.25};
  double cy2[6]={0.0,0.75,0.75,0.0,0.75,0.75};
  double sumi, sumy, N[6];

  
  for(sumy=0.0,ix=0;ix<NX;ix++)
    for(iy=0;iy<NY;iy++)
      {
	for(sumi=0.0,i=0;i<6;i++)
	  {
	    N[i]=RED(ix,iy,i)+BLUE(ix,iy,i);
	    sumi += (cy2[i]-cx2[i])*N[i];
	  }
	sumy += sumi;
      }

  sumy /= (double) NX;

  fprintf(stderr,"mech. surf. %f\t predicted %f\n",
	  0.5*sqrt(3.0)*sumy/2.0,-A*density/6.0*108.0/LAMBDA);
}

  
/*----------------------------------------------------------*/
  
double LaplaceLaw(int xc, int yc, double  R)  
{
/* R must be > 2 lattice units */
  int ix, iy, i, nr, nb;
  double dx, dy, p_red, p_blue, csq, r2, R2;

  R2 =  R*R;
  csq=0.5;

  nr=nb=0;
  p_red=p_blue=0.0;
  for(iy=0;iy<NY;iy++)
    for(ix=0;ix<NX;ix++)
      {
	dy = (double) (iy-yc);
	if(iy &1)
	  dx = (double) (ix-xc);
	else
	  dx = (double) (ix-xc)+0.5;
	if(dx*dx+dy*dy*3.0/4.0 < 0.6*R2)
	  {
	    for(i=0;i<6;i++)
	      p_blue += BLUE(ix,iy,i);
	    nb++;
	  }
	else if(dx*dx+dy*dy*3.0/4.0 > 1.4*R2)
	  {
	    for(i=0;i<6;i++)
	      p_red += RED(ix,iy,i);
	    nr++;
	  }
      }

  p_red *= csq/((double) nr);
  p_blue *= csq/((double) nb);
  
  return (-p_red+p_blue)*R;
}

