#ifndef BOLTZMANN_H
#define BOLTZMANN_H
#include "boltzmann.h"
#endif

/*#define TEST_STREAM */
#ifdef TEST_STREAM
/*------- temp --------*/
#include <stdio.h>
#define LAT(ix,iy,i) lat[((ix)+(iy)*NX)*6+(i)]
#endif

/*----------------------------------------------------------------------*
 *        (ix,iy+1) (ix+1,iy+1)                              *  *
 *              \       /                                    | /
 *               \     /                                     |/ 
 *  (ix-1,iy)----(ix,iy)----(ix+1,iy)      iy=odd         *--*--* 
 *                /    \                                     |\
 *               /      \                                    | \
 *        (ix,iy-1) (ix+1,iy-1)                              *  *
 *
 *
 *      (ix-1,iy+1) (ix,iy+1)                                *  *
 *              \       /                                     \ | 
 *               \     /                                       \|
 *  (ix-1,iy)----(ix,iy)----(ix+1,iy)      iy=even           *--*--*
 *                /    \                                       /|
 *               /      \                                     / |
 *        (ix-1,iy-1) (ix,iy-1)                              *  *
 * 
 *            /  2
 *            \  1
 *            /  0
 *
 *----------------------------------------------------------------------*/




void Stream(double *lat)
/* a very clumsy, but secure stream */
{
  int size, i, ix, iy;
  int xp1,yp1,xm1,ym1;
  

  size = NX*NY*6;
  
  for(i=0;i<size;i++)
    {
      temp[i]=lat[i];
      lat[i]=0.0;
    }

  for(iy=0;iy<NY;iy++)
    for(ix=0;ix<NX;ix++)
      {
	xp1=(ix+1+NX)%NX;
	yp1=(iy+1+NY)%NY;
	xm1=(ix-1+NX)%NX;
	ym1=(iy-1+NY)%NY;

	lat[(xp1+iy*NX)*6]=temp[(ix+iy*NX)*6+0];
	lat[(xm1+iy*NX)*6+3]=temp[(ix+iy*NX)*6+3];
	if(iy&1)
	  {
	    lat[(xp1+yp1*NX)*6+1]=temp[(ix+iy*NX)*6+1];
	    lat[(ix+yp1*NX)*6+2]=temp[(ix+iy*NX)*6+2];
	    lat[(ix+ym1*NX)*6+4]=temp[(ix+iy*NX)*6+4];
	    lat[(xp1+ym1*NX)*6+5]=temp[(ix+iy*NX)*6+5];
	  }
	else
	  {
	    lat[(ix+yp1*NX)*6+1]=temp[(ix+iy*NX)*6+1];
	    lat[(xm1+yp1*NX)*6+2]=temp[(ix+iy*NX)*6+2];
	    lat[(xm1+ym1*NX)*6+4]=temp[(ix+iy*NX)*6+4];
	    lat[(ix+ym1*NX)*6+5]=temp[(ix+iy*NX)*6+5];	
	  }
      }
}

/*----------------------------------------------------------------------*/

void Move(int ix, int iy, double* lat1, double* lat2)
{
  int xp1, yp1, xm1, ym1;

  xp1=(ix+1+NX)%NX;
  yp1=(iy+1+NY)%NY;
  xm1=(ix-1+NX)%NX;
  ym1=(iy-1+NY)%NY;

  lat2[(xp1+iy*NX)*6]=lat1[(ix+iy*NX)*6+0];
  lat2[(xm1+iy*NX)*6+3]=lat1[(ix+iy*NX)*6+3];
  if(iy&1)
    {
      lat2[(xp1+yp1*NX)*6+1]=lat1[(ix+iy*NX)*6+1];
      lat2[(ix+yp1*NX)*6+2]=lat1[(ix+iy*NX)*6+2];
      lat2[(ix+ym1*NX)*6+4]=lat1[(ix+iy*NX)*6+4];
      lat2[(xp1+ym1*NX)*6+5]=lat1[(ix+iy*NX)*6+5];
    }
  else
    {
      lat2[(ix+yp1*NX)*6+1]=lat1[(ix+iy*NX)*6+1];
      lat2[(xm1+yp1*NX)*6+2]=lat1[(ix+iy*NX)*6+2];
      lat2[(xm1+ym1*NX)*6+4]=lat1[(ix+iy*NX)*6+4];
      lat2[(ix+ym1*NX)*6+5]=lat1[(ix+iy*NX)*6+5];	
    }
}

/*----------------------------------------------------------------------*/


