
#ifndef BOLTZMANN_H
#define BOLTZMANN_H
#include "boltzmann.h"
#endif


void main()
/*---------------------------------------------------------------------*
   driver to run boltzmann
   commands read from script file. Each line in the script file 
   schould begin with a prefix listed below, 
   followed by a single space.
   prefixs are:
    '.':           indicates the end of the scrpt file
    'u':           set universe: read NX, NY
    'f':           output file name
    'n':           seed random generator
    's':           read solid and set universe
    'i':           initial flow param:1) prob_of_red,
                                      2) density
				      3) density fluct. in %
				      4) vx
				      5) vy
    'p':           set parameters : LAMBDA, perturb. of 
                                            surf. ten.
    'g':           set gravity
    'e':           experiment + 1) choice
                                2) frames
                                3) steps between each frame
    
 *------------------------------------------a---------------------------*/   

{
  char prefix, szInput[32];
  char szSolid[128], szOut[32];
  FILE* fPtrSolid;
  BOOL isRock=FALSE;


  int iframe, istep, steps;
  double vx, vy, A, gx, gy;
  char init_type, simul_type, output_type;


 /* read defaults values */
  fprintf(stderr,"*************************************************************\n");
  fprintf(stderr,"* 2D Boltzmann (BGK) method for immiscible flow.               *\n");
  fprintf(stderr,"* Author: Olav van Genabeek                                 *\n");
  fprintf(stderr,"*         Department of Earth Plan. and Atmos. Sciences     *\n");
  fprintf(stderr,"*         email: olav@segovia.mit.edu                       *\n");
  fprintf(stderr,"* Cambridge, March, 1997                                    *\n");
  fprintf(stderr,"*************************************************************\n");
  fprintf(stderr,"------------------------------------------------\n");
  fprintf(stderr,"PROGRAM ilb IS RUNNING NOW\n");
  seed = (long) time(NULL);  /* seed random gen */
  fprintf(stderr,"current date: %s",ctime(&seed));
  fprintf(stderr,"------------------------------------------------\n");
  fprintf(stderr,"INPUT PARAMETERS:\n");
  fprintf(stderr,"------------------------------------------------\n");
  seed = -seed;
  NX=NY=0;
  LAMBDA = -1;
  A=0.0;
  fluct=0.0;
  strcpy(szSolid,"\n");

/*start read input */
  scanf("%c",&prefix);
  while(prefix != '.')
    {
      switch(prefix)
	{
	case 'n':          /* seed random generator */
	  scanf(" %ld", &seed);
	  break;
	case 'f':          /* set finame for future outputs */
	  scanf(" %s",szOut);
	  break;
	case 'u':          /* set universe*/
 	  if(NX>0)
	    fprintf(stderr,"universe has been set\n");
	  scanf(" %d %d",&NX, &NY);
	  break;
	case 'p':   /*set relaxation constants: LAMBDA */
	  scanf(" %lf %lf", &LAMBDA, &A);
	  break;
	case 'g':
	  scanf(" %lf %lf", &gx, &gy);

	  break;
	case 's':          /* read solid */
	  scanf(" %s", szSolid);
	  break;
	case 'r':        
	  isRock=TRUE;

	  break;
	case 'i':          /* initializing parameters*/
	  {
	    fprintf(stderr,"random generator seeded with: %ld\n",seed);
	    scanf(" %c %lf %lf %lf %lf %lf", 
		  &init_type, &redprob,
		  &density, &fluct, &vx,&vy);
	    break;
	  }
	case 'e':
	  scanf(" %c %d %d",&simul_type, &FRAMES, &steps);
	  break;
	default:
	  /*	  gets(szInput);
	  fprintf(stderr,"unrecognized input command line:\n");
	  fprintf(stderr,"->%s\n",szInput); 
	  break; */
	}
      scanf("%c",&prefix);
    }
/* end input */

/*check dimensions */
  if(0==NX*NY)
    {
      fprintf(stderr,"invalid input dimensions %d x %d\n",
	      NX,NY);
      fprintf(stderr,"exiting....\n");
      exit(1);
    }

/* report input parameters */
  fprintf(stderr,"dimensions:      %dx%d\n",NX,NY);
  fprintf(stderr,"lamba:           %f ",LAMBDA);
  fprintf(stderr,"-> predic. visc.: %f\n", 
	  (-1/4.)*(1/LAMBDA+.5));
  fprintf(stderr,"probn. of red:   %f\n",redprob);
  fprintf(stderr,"density:         %f\n",density);
  fprintf(stderr,"fluctuation:     %f\n",fluct);
  fprintf(stderr,"              vx=%f vy=%f\n",vx,vy);
  fprintf(stderr,"gravity:      gx=%f gy=%f\n",gx,gy);
  fprintf(stderr,"surf. tens.      %f\n",A);
  fprintf(stderr,"------------------------------------------------\n");
/* initializing lattice */
  fprintf(stderr,"initializing lattice...");
  if(strlen(szSolid)>1)
    {
      if(!(fPtrSolid=fopen(szSolid, "r")))
	{
	  fprintf(stderr,"%s:%d cannot open file: %s\n",__FILE__,
		  __LINE__, szSolid);
	  exit(1);
	} 
      fread( (char *) &NX, sizeof(int), 1, fPtrSolid);
      fread( (char *) &NY, sizeof(int), 1, fPtrSolid);
      SetUniverse(NX, NY);
      if(!(fread( (char *) solid, sizeof(char), NX*NY,
		  fPtrSolid)))
	{
	  fprintf(stderr,"%s:%d cannot read file: %s", __FILE__,
		  __LINE__, szSolid);
	  exit(1);
	}
      fprintf(stderr,"reading solid file: %s ",szSolid);
      fprintf(stderr,"...Ok\n");
      fclose(fPtrSolid);
    }
  else
    SetUniverse(NX,NY);
  fprintf(stderr,"OK\n");

  if(TRUE == isRock)
    InitRock(solid);

  InitLat(init_type,redprob, density, fluct, vx, vy);
  TestCons(); 
  fprintf(stderr,"\n");

/* perform experiment */

  fprintf(stderr,"frames=%d steps=%d\n",
	       FRAMES, steps);
/* 
  simul_type == m: measure mechanical surface tension
             == r: Rayleigh Taylor instability
             == s: single phase flow thru porous media
             == t: compute viscosity by relaxation
*/
  switch(simul_type)
    {
    case 'b':
      for(iframe=0;iframe<FRAMES;iframe++) 
	{
	  for(istep=0;istep<steps;istep++)
	    {
	      Collision();
	      TwoPhase(solid,red,blue,A);
	      Stream(red); 
	      Stream(blue); 
	    }
	  TestCons(); 
	  fprintf(stderr,"%f %f\n", 
		  LaplaceLaw(NX/2, NY/2,(double) NX/4.0),
		  -A*density/6.0*108.0/LAMBDA);
	}
      break;      
    case 'm':
      fprintf(stderr,"measure mechanical surface tension\n",
	      init_type);
      fprintf(stderr,"------------------------------------------------\n");
      for(iframe=0;iframe<FRAMES;iframe++) 
	{
	  for(istep=0;istep<steps;istep++)
 	    {
	      Stream(red); 
	      Stream(blue); 
	      Collision(); 
	      TwoPhase(solid,red,blue,A);

	    }
	  TestCons(); 
	  MechSurfTension(A);
	}
	  /*		    WriteVel(szOut);  */
      break;
    case 'u':
      for(iframe=0;iframe<FRAMES;iframe++) 
	{
	  for(istep=0;istep<steps;istep++)
	    {
	      Collision(); 
	      TwoPhase(solid,red,blue,A);
	      Stream(red); 
	      Stream(blue); 
	    }
	  TestCons(); 
	}
      break;
	  /*		    WriteVel(szOut);  */
    case 'r':
      { 
	int ix, iy, i;
	double b, r;

	for(iframe=0;iframe<FRAMES;iframe++) 
	  {
	    for(istep=0;istep<steps;istep++)
	      {
		Collision(); 
		TwoPhase(solid,red,blue,A);
		ForceEqual(gx, gy); 
		Stream(red); 
		Stream(blue); 
	      }
	    TestCons();   
/* write height of interface to std. io */
	    for(ix=0;ix<NX;ix++)
	      for(iy=0;iy<NY;iy++)
		{
		  for(i=0,r=b=0.0;i<6;i++)
		    {
		      r += RED(ix,iy,i);
		      b += BLUE(ix,iy,i);
		    }
		  if(r<b)
		    {
		      printf("%d %d\n",ix,iy);
		      iy=NY;
		    }
		}
	  }
      }      
      break;
    case 's':
      for(iframe=0;iframe<FRAMES;iframe++) 
	{
	  for(istep=0;istep<steps;istep++)
	    {
	      Collision(); 
	      AddGravity(red,gx,gy); 
	      Stream(red); 
	      Stream(blue); 
	    }
	  TestCons();   
	  ChanVisc(gx);
	}
	  /*		    WriteVel(szOut);  */
      break;

    }                                              
  CleanUniverse();
}


