/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#include "include_ilg.h"
#include "parameters_ilg.h"

main()
{
  char *col_dens,*lat_out;
  char colorsum[N_LCONF_CODES];
  unsigned short *lattice,*nlattice,*ptr;                
  unsigned short *enum_class[N_CONF_CLASSES];
  unsigned short enum_class_data[N_CONF+N_CONF_CLASSES];
  unsigned short *tie_list[N_CONF_CLASSES][N_ANGLE_CODES];
  unsigned short tie_list_data[TIE_LIST_SIZE];
  int i,nx,ny,nz,seed,it,nt,dt;
  int angle[X_FIELD_RANGE][Y_FIELD_RANGE];
  int conf_class[N_LCONF_CODES];
  float density,red_prob;
  FILE *outfd,*history_file_fd;

  /* "read" parameters */
  nx=NX; 
  ny=NY;
  nt=NT;
  dt=DT;
  density=DENSITY;
  red_prob=RED_PROB;

  /* open output file for movie */
  if ( (outfd = fopen("ilgout","w")) == NULL)
  {
    fprintf(stderr,"Could not open file -- ilgout --. \n");
    exit(1);
  }

  /* open and write output file that says where movie is located, 
     and gives its size (n1=generic fast dimension, n2=next fastest, etc) */
  history_file_fd = fopen("history","w");
  fprintf (history_file_fd,"in=ilgout\n");
  fprintf (history_file_fd,"n1=%d\n",nx);
  fprintf (history_file_fd,"n2=%d\n",ny);
  fclose(history_file_fd);

  /* Calculate the length of the lattice. */
  nz = nx*ny;

  /* Allocate memory for the arrays. */
  lattice = (unsigned short *) malloc(nz*sizeof(short));
  nlattice = (unsigned short *) malloc(nz*sizeof(short));
  col_dens = (char *) malloc(nz*sizeof(char));
  lat_out = (char *) malloc(nz*sizeof(char));

  /* initialize random number generator */
  seed = time(0);
  srandom(seed);

  /* initialize lookup tables */
  init_colorsum(colorsum);
  init_conf_class(conf_class,enum_class,enum_class_data);
  init_angle(angle);
  init_tie_list(tie_list,tie_list_data,enum_class);

  /* initialize lattice */
  init_lat(lattice,density,red_prob,nz);

  /* output message with number red, blue, and x and y momentum */
  fprintf(stderr,"state of lattice at initialization:\n");
  test_cons(lattice,nz); 

  /* basic lattice update and output */
  for (it=0; it<nt; it++)
  {
    /* output every dt time steps, including initial state */
    if (it%dt==0)
    {
      output_lat(lattice,lat_out,colorsum,nz);
      fwrite( (char *)lat_out, sizeof(char), nz*sizeof(char), outfd);
    }

    /* collisions and propagation */
    update_lat(nx,ny,lattice,nlattice,col_dens,
               colorsum,tie_list,conf_class,angle);

    /* exchange pointers of input and output lattices */
    ptr = lattice;
    lattice = nlattice;
    nlattice = ptr;
  }

  /* confirm conservation of mass, momentum, and color */
  fprintf(stderr,"state of lattice at termination:\n");
  test_cons(lattice,nz);
}
