/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
/* Create the table angle[xcolor][ycolor] 

   If the components of the external color field are given by  
   xcolor and ycolor, where 

	-X_FIELD_RANGE/2 <= xcolor <= X_FIELD_RANGE/2
	-Y_FIELD_RANGE/2 <= ycolor <= Y_FIELD_RANGE/2

   then the angle number for that field is given by

   	angle[xcolor][ycolor].

   Angle numbers for non-zero fields range from 0 to NANGLES-1.
   Zero fields are denoted by angle number = ZERO_FIELD.
*/

#include "include_ilg.h"

init_angle(angle)
int angle[X_FIELD_RANGE][Y_FIELD_RANGE];
{
  int i,xcolor,ycolor,x;
  float xtrue,ytrue,angle_true,twopi,unit_angle,sqrt3;

  sqrt3 = sqrt(3.0);
  twopi = 2.*PI;
  unit_angle = twopi / N_ANGLES;

  i = 0;
  for(xcolor=0; xcolor<X_FIELD_RANGE; xcolor++)
    for(ycolor=0; ycolor<Y_FIELD_RANGE; ycolor++)
    {
      /* restore negativity to x- and ycolor, and convert to floats */
      xtrue = xcolor - X_FIELD_RANGE/2;
      ytrue = sqrt3 * ( ycolor - Y_FIELD_RANGE/2 );

      /* Create angle[xcolor][ycolor].
       * angle[xcolor][ycolor]=ZERO_FIELD signifies zero-field. */

      if (xtrue==0 && ytrue==0)
          angle[xcolor][ycolor] = ZERO_FIELD; 
      else
      {
      	  angle_true = atan2((double)ytrue,(double)xtrue); /* range -pi to pi */

	  if ( angle_true>=0 )
	    angle[xcolor][ycolor] = angle_true/unit_angle + .5;
          else
	    angle[xcolor][ycolor] = 
	    (int) ( (angle_true+twopi)/unit_angle + .5 ) % N_ANGLES;
	    			/* modulo to prevent wrong roundoff */
      }
    }

#ifdef DIAGNOSTICS
  for (xcolor=0; xcolor<X_FIELD_RANGE; xcolor+=4)
    for (ycolor=0; ycolor<Y_FIELD_RANGE; ycolor+=2)
      fprintf(stderr, "xcolor=%d, ycolor=%d, angle_code=%d\n",
      xcolor-X_FIELD_RANGE/2,ycolor-Y_FIELD_RANGE/2,
      angle[xcolor][ycolor]);
#endif

}
