/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
/*  Form the following tables:

	conf_class[lconf] gives the configuration class (1,..,360)
	for a given 14 bit lconf.

	*enum_class[conf_class] gives the number n of lconfs in 
	configuration class conf_class.

	enum_class[conf_class][1,...,n] gives the lconfs that are
	in configuration class conf_class.
*/


#include "include_ilg.h"

init_conf_class(conf_class,enum_class,enum_class_data)
unsigned short *enum_class[N_CONF_CLASSES];
unsigned short enum_class_data[N_CONF+N_CONF_CLASSES];
int conf_class[N_LCONF_CODES];
{
  int lconf,class,class_count;
  int n,r,px,py;
  int aa,bb,cc,dd,ee,ff,gg;
  unsigned short *ptr;
  unsigned short classes[N_VEL+1][N_VEL+1][PX_RANGE][PY_RANGE];
  /* classes[n][r][px][py]
     n = particle number
     r = number or red particles
     px = number of possible x-momentum
     py = number of possible y-momentum
  */

  /* initialize classes[] */
  bzero((char *)classes, (N_VEL+1)*(N_VEL+1)*PX_RANGE*PY_RANGE*sizeof(short));

  /* Loop through only the real lconfs, skipping the invalid codes. */
  for(aa=0; aa<3; aa++)
    for(bb=0; bb<3; bb++)
      for(cc=0; cc<3; cc++)
        for(dd=0; dd<3; dd++)
          for(ee=0; ee<3; ee++)
            for(ff=0; ff<3; ff++)
              for(gg=0; gg<3; gg++)
	      {
		lconf = ENCODE_LCONF(aa,bb,cc,dd,ee,ff,gg);
		n = PARTSUM(lconf);
		r = REDSUM(lconf);
		px = X_MOM(lconf) + PX_RANGE/2;
		py = Y_MOM(lconf) + PY_RANGE/2;

		/* Increment the size of the class lconf belongs to. */
		classes[n][r][px][py]++;
	      }

  class_count = 0; 
  ptr=enum_class_data;
  for(n=0; n<=N_VEL; n++)
    for(r=0; r<=n; r++)
      for(px=0; px<PX_RANGE; px++)
        for(py=0; py<PY_RANGE; py++)
	  if(classes[n][r][px][py])
	  {
            /* Store the pointer into enum_class_data in enum_class[]. */
	    enum_class[class_count] = ptr;

	    /* Initialize enum_class[class_count][0]. */
	    *ptr = 0;

    	    /* Increase ptr to make room for the lconfs in the class. */
	    ptr += classes[n][r][px][py]+1;

  	    /* Store the code class_count back in classes[] where
	     * we read the size of the class; increment classcount. */
	    classes[n][r][px][py] = class_count++;
	  }

  /* Loop through only the real lconfs, skipping the invalid codes. */
  /* Plug the conf codes into conf_class where they belong. */
  for(aa=0; aa<3; aa++)
    for(bb=0; bb<3; bb++)
      for(cc=0; cc<3; cc++)
        for(dd=0; dd<3; dd++)
          for(ee=0; ee<3; ee++)
            for(ff=0; ff<3; ff++)
              for(gg=0; gg<3; gg++)
	      {
		lconf = ENCODE_LCONF(aa,bb,cc,dd,ee,ff,gg);
		n = PARTSUM(lconf);
		r = REDSUM(lconf);
		px = X_MOM(lconf) + PX_RANGE/2;
		py = Y_MOM(lconf) + PY_RANGE/2;

		class = classes[n][r][px][py];	/* get the conf_class */

		conf_class[lconf] = class;	/* form conf_class */

	        /* get the ptr into enum_class_data[] */
		ptr = enum_class[class];

		/* increment the record of the class size
		 * (stored at ptr[0] = enum_class[class][0])
		 * and store the lconf code in the right place. */
		ptr[++(ptr[0])] = lconf;
	      }

#ifdef DIAGNOSTICS
  aa=0;
  for(bb=0; bb<N_CONF_CLASSES; bb++)
  {
    aa += *enum_class[bb];
    fprintf(stderr,"class=%d	size=%d	running total=%d\n",bb,*enum_class[bb],aa);
  }
#endif
}
