/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
/*  Form the tie list of candidates for collision results:

	*tie_list[conf_class][angle_code] gives the number n of lconfs 
	that are candidates for the collision outcome.

	tie_list[conf_class][angle_code][1,...,n] contains these n lconfs.
*/

#include "include_ilg.h"

init_tie_list(tie_list,tie_list_data,enum_class)
unsigned short *tie_list[N_CONF_CLASSES][N_ANGLE_CODES];
unsigned short tie_list_data[TIE_LIST_SIZE];
unsigned short *enum_class[N_CONF_CLASSES];
{
  int i,lconf,conf_class,angle_code,class_size,nties;
  unsigned short *pnties,*plconf;
  float eps=.000001;  /* added to avoid roundoff error problems */
  float max, result[MAX_CLASS_SIZE];
  double sqrt3,xflux,yflux;
  double angle_true,twopi,unit_angle,xc[N_ANGLE_CODES],yc[N_ANGLE_CODES];

  /* First calculate cartesian coordinates for each of the 
   * non-zero-field angle codes.			*/

  sqrt3 = sqrt(3.0);
  twopi = 2.*PI;
  unit_angle = twopi / N_ANGLES;
  for(angle_code=0; angle_code<N_ANGLES; angle_code++)
  {
    angle_true = angle_code*unit_angle;
    xc[angle_code] = cos( (double)angle_true );
    yc[angle_code] = sin( (double)angle_true );
  }
  /* set the x and y coordinates to 0 for case of zero field */
  xc[N_ANGLES] = yc[N_ANGLES] = 0;

  /* Now form tie_list.
   * Loop over all possible conf_classes and angle_codes.
   * Calculate result of flux dotted with external field (angle) */

  pnties = tie_list_data;
  for(conf_class=0; conf_class<N_CONF_CLASSES; conf_class++)
  {
    for(angle_code=0; angle_code<N_ANGLE_CODES; angle_code++)
    {
      class_size = *enum_class[conf_class];

      /* calculate product of flux and field vector.*/
      for(i=0;i<class_size;i++)
      {
        lconf = enum_class[conf_class][i+1];
        xflux = X_FLUX(lconf);
        yflux = sqrt3 * Y_FLUX(lconf);
	result[i] = xflux*xc[angle_code] + yflux*yc[angle_code];
      }

      /* What is the maximum result for this angle_code? */
      for(i=0,max=result[0]; i<class_size; i++)
      {
	if( result[i] > max )
	  max = result[i];
      }

      /* How many results are as large as max? 
       * Place this number n in tie_list[lconf][angle_code][0].
       * Place each of these lconfs in tie_list[conf_class][angle_code][i+1]  */

      plconf = pnties + 1;
      for(i=0,nties=0;i<class_size;i++)
      {
	if( result[i] < max+eps && result[i]>max-eps )
	{
          lconf = enum_class[conf_class][i+1];
	  *plconf = lconf;  
	  plconf++;
	  nties++;
        }
      }

      *pnties = nties;
      tie_list[conf_class][angle_code] = pnties;
      pnties += nties+1;
    }
  }  

#ifdef DIAGNOSTICS
  fprintf(stderr,"total=%d\n",plconf-tie_list_data);
  /*
  for(conf_class=0; conf_class<N_CONF_CLASSES; conf_class+=4)
  {
    for(angle_code=0; angle_code<N_ANGLE_CODES; angle_code+=4)
    {
      fprintf(stderr,"class=%d, angle=%d, n_in_class=%d, nties=%d\n",
      conf_class,angle_code,
      *enum_class[conf_class],
      *tie_list[conf_class][angle_code]);
    }
  }

  maxties = 0;
  for(conf_class=0; conf_class<N_CONF_CLASSES; conf_class++)
  {
    for(angle_code=0; angle_code<N_ANGLE_CODES; angle_code+=4)
    {
      nties = tie_list[conf_class][angle_code][0];
      if (nties>maxties)
        maxties = nties;
    }
  }
  fprintf(stderr,"maxties=%d\n",maxties);
  */
#endif
}
