/* Velocities and cartesian axes are in the following directions:

                                                  y
                    C             B              / \
                      \        /                  |
                        \    /                    |
              D   ------  G   ------  A           -----> x
                        /    \
                       /      \
                    E            F

*/
/*  These macros extract the 2-bit fields for each velocity */
#define A_EXTR(x)  (x&A_MASK)
#define B_EXTR(x) ((x&B_MASK)>>2)
#define C_EXTR(x) ((x&C_MASK)>>4)
#define D_EXTR(x) ((x&D_MASK)>>6)
#define E_EXTR(x) ((x&E_MASK)>>8)
#define F_EXTR(x) ((x&F_MASK)>>10)
#define G_EXTR(x) ((x&G_MASK)>>12)

/* Determine if a particle is present */
#define PART(y) ((y==RED)+(y==BLUE))
#define AP(x) PART(A_EXTR(x))
#define BP(x) PART(B_EXTR(x))
#define CP(x) PART(C_EXTR(x))
#define DP(x) PART(D_EXTR(x))
#define EP(x) PART(E_EXTR(x))
#define FP(x) PART(F_EXTR(x))
#define GP(x) PART(G_EXTR(x))

/* The x-momentum times 2 */
#define X_MOM(x) ((int) (2*AP(x) - 2*DP(x) + BP(x) + FP(x) - EP(x) - CP(x) ))

/* The y momentum times 2/sqrt (3 ) */
#define Y_MOM(x)  ( (int)(BP(x) + CP(x) -  EP(x) - FP(x)) )

/* This generates an 14-bit configuration from the components. */
#define ENCODE_LCONF(a,b,c,d,e,f,g)	\
	(a|(b<<2)|(c<<4)|(d<<6)|(e<<8)|(f<<10)|(g<<12))

/* Determine sign of color: RED is positive, BLUE is negative */
#define COLR(y) ((y==RED)-(y==BLUE))
#define A_COLR(x) COLR(A_EXTR(x))
#define B_COLR(x) COLR(B_EXTR(x))
#define C_COLR(x) COLR(C_EXTR(x))
#define D_COLR(x) COLR(D_EXTR(x))
#define E_COLR(x) COLR(E_EXTR(x))
#define F_COLR(x) COLR(F_EXTR(x))
#define G_COLR(x) COLR(G_EXTR(x))

/* Determine if a red particle is present */
#define RED_P(y) (y==RED)
#define A_RED_P(x) RED_P(A_EXTR(x))
#define B_RED_P(x) RED_P(B_EXTR(x))
#define C_RED_P(x) RED_P(C_EXTR(x))
#define D_RED_P(x) RED_P(D_EXTR(x))
#define E_RED_P(x) RED_P(E_EXTR(x))
#define F_RED_P(x) RED_P(F_EXTR(x))
#define G_RED_P(x) RED_P(G_EXTR(x))

/* Twice the colored (red-blue) x-momentum */
#define X_FLUX(x) ( 2*A_COLR(x) - 2*D_COLR(x) + B_COLR(x) \
                    + F_COLR(x) - E_COLR(x) - C_COLR(x) )

/* 2/sqrt times the colored (red-blue) y-momentum */
#define Y_FLUX(x) ( B_COLR(x) + C_COLR(x) - E_COLR(x) - F_COLR(x) )


/* This calculates the sum of signed color at a site */
#define COLRSUM(x) \
  (A_COLR(x)+B_COLR(x)+C_COLR(x)+D_COLR(x)+E_COLR(x)+F_COLR(x)+G_COLR(x))

/* This calculates the number of particles at a site. */
#define PARTSUM(x) \
  (AP(x)+BP(x)+CP(x)+DP(x)+EP(x)+FP(x)+GP(x))

/* This calculates the number of RED particles at a site. */
#define REDSUM(x) \
  (A_RED_P(x)+B_RED_P(x)+C_RED_P(x)+D_RED_P(x)+E_RED_P(x)+F_RED_P(x)+G_RED_P(x))

/* Some possibly useful mathish macros */
#define WRAP_DN(p,n)    (((p) >= (n)) ? ((p)-(n)) : (p))
#define WRAP_UP(p,n)    (( (p) < 0) ? ((p)+(n)) : (p))
#define WRAP_AROUND(p,n)        (( (p) < 0) ? ((p)+(n)) : WRAP_DN(p,n))

