/*
Lgapack Version VERSION for the simulation of flow with lattice-gas automata. 
Copyright (C) 1997 D.H. Rothman and S. Zaleski. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
in the file GNU_General_Public_License, along with this program; 
if not, write to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

dan@segovia.mit.edu, zaleski@lmm.jussieu.fr

 Stephane Zaleski
 LMM, UPMC, Tour 66 Case 162  
 4 place Jussieu           	
 75252 PARIS Cedex 05  
 FRANCE                

 Daniel H. Rothman
 Dept of EAPS, 54-625
 MIT
 Cambridge Ma 02139
 USA
*/
#include "include_ilg.h"

update_lat(nx,ny,lattice,nlattice,col_dens,colorsum,tie_list,conf_class,angle)
int nx,ny;
unsigned short *lattice,*nlattice;
char colorsum[N_LCONF_CODES];
char *col_dens;
unsigned short *tie_list[N_CONF_CLASSES][N_ANGLE_CODES];
int conf_class[N_LCONF_CODES];
int angle[X_FIELD_RANGE][Y_FIELD_RANGE];
{
  unsigned short lconf,olconf,ix,iy,*ptr;
  int nz,iz,za,zb,zc,zd,ze,zf;
  int iy_b,iy_c,iy_e,iy_f,diff_be,diff_cf,angx,angy;
  int angle_code,iym1,iyp1,ixm1,ixp1;

  nz = nx*ny;

  /* calculate the color density at each site; also zero the target array */
  for(iz=0; iz<nz; iz++)
  {
    col_dens[iz] = colorsum[ lattice[iz] ];
    nlattice[iz] = 0;
  }

  /* Loop over y-axis (slow dimension) */
  for(iy=0; iy<ny; iy++)
  {
    /* deterimine y-index of neighbors in directions b,c,d,f */
    iym1 = iy - 1;
    iyp1 = iy + 1;
    iy_b = WRAP_AROUND(iyp1,ny);
    iy_c = iy_b;
    iy_e = WRAP_AROUND(iym1,ny);
    iy_f = iy_e;

    /* loop over x-axis (fast dimension) */
    for(ix=0; ix<nx; ix++)
    {
      iz = iy*nx + ix;
      ixm1 = ix - 1;
      ixp1 = ix + 1;

      za = iy*nx + WRAP_AROUND(ixp1,nx);
      zd = iy*nx + WRAP_AROUND(ixm1,nx);

      /* if odd-numbered row */
      if (iy&1)
      {
        zb = iy_b*nx + WRAP_AROUND(ixp1,nx);
        zf = iy_f*nx + WRAP_AROUND(ixp1,nx);
        ze = iy_e*nx + ix;
        zc = iy_c*nx + ix;;
      }
      else
      {
        zb = iy_b*nx + ix;
        zf = iy_f*nx + ix;
        ze = iy_e*nx + WRAP_AROUND(ixm1,nx);
        zc = iy_c*nx + WRAP_AROUND(ixm1,nx);
      }

      olconf = lattice[iz];

      /* Determine angle of the color gradient */
      diff_be = col_dens[zb] - col_dens[ze];
      diff_cf = col_dens[zc] - col_dens[zf];
      angx = 2*(col_dens[za]-col_dens[zd]) + diff_be-diff_cf + X_FIELD_RANGE/2;
      angy = diff_be + diff_cf + Y_FIELD_RANGE/2;
      angle_code = angle[angx][angy];

      /* Find pointer to possible candidates for the collision */
      ptr = tie_list[conf_class[olconf]][angle_code];

      /* only 1 possible result for the collision */
      if(ptr[0]==1)
        lconf = ptr[1];

      /* otherwise chooose randomly among equally good results */
      else
        lconf = ptr[((random())%ptr[0])+1];

      /* propagate result of collision to neighboring sites */
      nlattice[za] |= lconf & A_MASK;
      nlattice[zb] |= lconf & B_MASK;
      nlattice[zc] |= lconf & C_MASK;
      nlattice[zd] |= lconf & D_MASK;
      nlattice[ze] |= lconf & E_MASK;
      nlattice[zf] |= lconf & F_MASK;
      nlattice[iz] |= lconf & G_MASK;
    }
  }
}
