/*<<<<<<<<<<<<<<<<<<<DRIVER for BGK>>>>>>>>>>>>>>>>>*/
#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif

void main()
/*---------------------------------------------------------------------*
   driver to run BGK
   commands read from script file. Each line in the script file 
   schould begin with a prefix listed below, followed by a single
   space.
    prefixs are:
    '.':           indicates the end of the scrpt file
    'f':           output file name
    'g':           gravity  
    'r':           seed random generator
    'm':           select FCHC model and set the direction vectors
    's':           read solid and set universe
    'i':           initialize lattice
    'c':           channel
    'p':           read two parameters lambda_Nu, lambda_D
 *------------------------------------------a---------------------------*/   

{
  char prefix;
  char szModel[128];
  FILE* fPtrSolid;
  BOOL bIsParam = FALSE;

  int alpha, i;
  int ix, iy, iz;
  float vx, vy, vz;

 /* read defaults values */
  fprintf(stderr,"*************************************************************\n");
  fprintf(stderr,"* 3D Boltzmann (BGK)                      *\n");
  fprintf(stderr,"* Author: Olav van Genabeek                                 *\n");
  fprintf(stderr,"*         Department of Earth Plan. and Atmos. Sciences     *\n");
  fprintf(stderr,"*         email: olav@segovia.mit.edu                       *\n");
  fprintf(stderr,"* Cambridge, March, 1997                                    *\n");
  fprintf(stderr,"*************************************************************\n");
  seed = (long) time(NULL);                           /* seed random gen */
  fprintf(stderr,"current date: %s",ctime(&seed));
  seed = -seed;

  scanf("%c",&prefix);
  while(prefix != '.')
    {
      switch(prefix)
	{
	case 'm':
	  scanf(" %s", szModel);
	    if(!strcmp("D3Q19",szModel))
	      {
		fprintf(stderr,"using %s model\n",szModel);
		SetD3Q19();
/*		for(i=0;i<19;i++)
		  fprintf(stderr,"%5d%3d%3d%3d\n",
			  i,C(i,0),C(i,1),C(i,2)); */

	      }
	  break;
	case 'r':          /* seed random generator */
	  scanf(" %ld", &seed);
	  break;
	case 'f':          /* set finame for future outputs */
	  scanf(" %s", szOut);
	  break;
	case 'g':          /* gravity */
	  scanf(" %f", &gravity);
	  break;
	case 'u':          /* set universe, cannot be used after switch s!! */
	  scanf(" %d %d %d",&Nx, &Ny, &Nz);
	  fprintf(stderr,"dimensions %dx%dx%d\n",Nx,Ny,Nz);
	  SetUniverse(Nx,Ny,Nz);
	  break;
	case 'p':          /* set relaxation constants: lambda_D, lambda_Nu */
	  scanf(" %f", &lambda_Nu);
	  fprintf(stderr,"viscosity:   %f\n", (-1/3.)*(1/lambda_Nu+.5));
	  bIsParam=TRUE;
	  break;
	case 's':          /* read solid */
	  scanf(" %s", szSolid);
	  if(!(fPtrSolid=fopen(szSolid, "r")))
	    {
	      fprintf(stderr,"%s:%d cannot open file: %s\n",__FILE__,
		      __LINE__, szSolid);
	      exit(1);
	    } 
	  fread( (char *) &Nx, sizeof(int), 1, fPtrSolid);
	  fread( (char *) &Ny, sizeof(int), 1, fPtrSolid);
	  fread( (char *) &Nz, sizeof(int), 1, fPtrSolid);
	  fprintf(stderr,"dimensions %dx%dx%d\n",Nx,Ny,Nz);
	  SetUniverse(Nx, Ny, Nz);
	  if(!(fread( (char *) solid, sizeof(char), Nx*Ny*Nz,
		     fPtrSolid)))
	    {
	      fprintf(stderr,"%s:%d cannot read file: %s", __FILE__,
		      __LINE__, szSolid);
	      exit(1);
	    }
	  fprintf(stderr,"reading solid file: %s ",szSolid);
	  fprintf(stderr,"...Ok\n");
/*	  for(iz=0;iz<Nz;iz++)
	    for(iy=0;iy<Ny;iy++)
	      for(ix=0;ix<Nx;ix++)
		fprintf(stderr,"%5d %5d %d\n",ix, iz, SOLID(ix,iy,iz)); */
	  fclose(fPtrSolid);
	  break;
	case 'i':          /* initializing lattice, careful: universe should be set!! */
	  {
	    fprintf(stderr,"random generator seeded with: %ld\n",seed);
	    scanf(" %f", &density);
	    fprintf(stderr,"density:         %f\n",density);
	    fprintf(stderr,"initializing lattice \n");
	    InitLat(density, 0);
	    TestCons(); 
	    InitPropagate();
	    fprintf(stderr,"\n");
	  }
	  break;
	case 'c':
	  {
/* read: parameters: lambda_Nu and lambda_D
   number of frames, steps */
	    int iframe, frames, istep, steps;
	    char mode, writemode;

	    if(bIsParam == FALSE)
	      {
		fprintf(stderr,"need parameters\n");
		break;
	      }
	    else
	      {
		scanf(" %c %d %d %c",&mode, &frames, &steps, &writemode);	    
		fprintf(stderr,"frames=%d steps=%d\n",frames, steps);

/* 
   mode == v: write velocities
*/
		switch(mode)
		  {
		  case 'v': /* should be extended */
/*		WriteVel(szOut,mode); */
		    for(iframe=0;iframe<frames;iframe++) 
		      {
			for(istep=0;istep<steps;istep++)
			  {
			    Propagate(n);
			    Relaxate();  
			    AddGravity(density, gravity);
			  }
			TestCons();   
		      }
		    WriteVel(szOut,writemode);  
		    break;
		  }       /* end of writemode switch */
	      }
	  }
	  break;
	}
      scanf("%c",&prefix);
    } /*end while */

  CleanUniverse();
  free(map); /*!!!!! needs to be included in both set 
	       and clean universe routines !!!!!*/
}








