#ifndef __BOLTZMANN_H
#define __BOLTZMANN_H
#include "boltz.h"
#endif


#ifdef DEAD
/***********************************************************************/

int Weight(int side,int L)
{
/*---------------------------------------------------------------------*
 * Weigth function to correct for the FCHC projection:
 * the double particle axes indexed 12 to 17 should be
 * counted twice.
 *---------------------------------------------------------------------*/
  if(side < 12)
    return 1;
  else if(side < 19 - 1)
    return 2;
  else
    return L;
}
#endif

/***********************************************************************/
float Rho(float* N)
{
/*---------------------------------------------------------------------*
 * returns the density of the configuration
 *---------------------------------------------------------------------*/
  int i;
  float rho;

  rho= 0;

  for(i = 0;i<19;i++)   
    rho+=N[i];
  return (float) rho;
}

/***********************************************************************/

void GetV(para_tag* para, float* node, float* u)
{
/*---------------------------------------------------------------------*
 * computes the velocity corresponding to one node
 *---------------------------------------------------------------------*/
  int j;
  int alpha;
  float rho;

  rho = Rho(node);

  for(alpha=0; alpha<3; alpha++)
    { 

      u[alpha]=0.0;
      for(j=0; j<19-1; j++)
	u[alpha] += node[j]*C(j,alpha);
      u[alpha] /= rho; 
    }
}


/**********************************************************************/

void GetNeq(para_tag* para, float rho, float* u, float* Neq)
{
/*---------------------------------------------------------------------*
 * GetNeq
 * computes the equilibrium part of the lattice-boltzmann operator
 *---------------------------------------------------------------------*/
   int i,alpha, beta;
   float G;
   float sum1, sum2;

   for (i = 0; i < 19 - 1; i++) 
     {
       sum1 = sum2 =  0.0;
       for ( alpha = 0; alpha < 3; alpha++)
	 {
	   sum1 += (float) C(i,alpha)*u[alpha]/para->CsSq;
	   for ( beta = 0; beta < 3; beta++)
	     sum2+=para->G*(Q(i,alpha,beta)-(0.5-para->CsSq)*
			    delta(alpha,beta))*u[alpha]*u[beta];
	 }
       Neq[i]=para->d*(1+2*sum1+sum2);
/*       Neq[i]=para->d*(2*sum1+sum2);
       Neq[i]+=rho/(para->L+24);  */
     }
/* rest-particle */
/*   Neq[18]= rho/(para->L+24); */
   Neq[18] = para->d; 
   Neq[18] -= para->d*(para->CsSq*para->G
		    *(u[0]*u[0]+u[1]*u[1]+u[2]*u[2]) );

 }

/**********************************************************************/

void Collide(para_tag* para, var_tag* var)
{
/*--------------------------------------------------------------------*
 * computes the collision:
 * computation in three steps:
 *  a) appy forcing
 *  b) determine the new equilibrium state Neq which is a function of
 *     the local velocity u
 *  c) N(x,t) = O(i,j)*(N(x+C(j),t)-Neq(u(x,t)))
 *  in propagation step set N(x,t)->N(x+C(i),t+1)
 *--------------------------------------------------------------------*/
  int j,i;
  int ix,iy,iz;
  int alpha;
  float u[3];
  float Nold[19], Neq[19];
  float sum;
  float rho;

  for(iz=0;iz<para->Nz;iz++)
    for(iy=0;iy<para->Ny;iy++)
      for(ix=0;ix<para->Nx;ix++)
/*--------------------------------------------------------------------*
 * apply collision rules:
 *  -  in volume use the linearized Boltzmann operator
 *  -  @ boundaries use the no-slip or bounce rules 
 *--------------------------------------------------------------------*/
	{
	  for(j=0;j<19;j++)
	    {
/*--------------------------------------------------------------------*
 * add force and
 * update the side velocity with +F/2 to correct
 * for the boundary conditions (see Ginzburg et. al. (1994))
 *--------------------------------------------------------------------*/
	      LATTICE(ix,iy,iz,j)+=var->dN[j];	      
	      Nold[j]=LATTICE(ix,iy,iz,j);
	    }
	  rho=Rho(Nold);
	  GetV(para,Nold,u);
	  for(alpha = 0; alpha < 3; alpha++)
	    u[alpha] -= para->F[alpha]/2;
	  GetNeq(para,rho,u,Neq);

	  for(i=0;i<19;i++)
	    {
/* correct for the sides w/ 2 particles and the side w/ the rest part.*/
	      sum=0.0;
	      for(j=0;j<12;j++)
		sum += O(i,j)*(Nold[j]-Neq[j]);
	      for(j=12;j<18;j++)
		sum += O(i,j)*(Nold[j]-Neq[j]*2);		
	      sum +=  O(i,18)*(Nold[18]-Neq[18])*para->L;	
	      LATTICE(ix,iy,iz,i) = sum + Nold[i];
	    }
/*--------------------------------------------------------------------*
  table of velocties and hence the order of {n[i]}has been chosen
  such that the bounce rules are easy to implement. 
 *--------------------------------------------------------------------*/
	  if(SOLID(ix,iy,iz))
	    {
	      for(j=0;j<19;j++)
		Nold[j]=LATTICE(ix,iy,iz,j);
/*--------------------------------------------------------------------*
 * bounce rules for single particles 
 *--------------------------------------------------------------------*/
	      for( i = 0; i < 6; i++)
		{
		  LATTICE(ix,iy,iz,i) = Nold[i+6];
		  LATTICE(ix,iy,iz,i+6) = Nold[i];
		}
/*--------------------------------------------------------------------*
 * bounce rules for axis w/ max. 2 particles 
 *--------------------------------------------------------------------*/
	      for(i = 12; i < 15; i++)
		{
		  LATTICE(ix,iy,iz,i) = Nold[i+3];
		  LATTICE(ix,iy,iz,i+3) = Nold[i];
		}
	    }
	}
}
	      

	  
	    



