#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif


void Inject(void)
/*-----------------------------------------------------*
   recolors 100% blue at the entrance of the channel
   only particle with velocity component in the
   positive z direction are recolored
   That is only components: 1, 2, 10, 11, 14
 *-----------------------------------------------------*/

{
  int ix, iy, iz, j;
  float red, blue;
 

  for(iy=0;iy<Ny;iy++)
    for(ix=0;ix<Nx;ix++)
      {
	iz=0;                 /* inject at iz = 0; */
	blue = N(ix,iy,iz,1);
	DELTA(ix,iy,iz,1) = - blue; 
	blue = N(ix,iy,iz,2);
	DELTA(ix,iy,iz,2) = - blue; 
	blue = N(ix,iy,iz,10);
	DELTA(ix,iy,iz,10) = - blue; 
	blue = N(ix,iy,iz,11);
	DELTA(ix,iy,iz,11) = - blue; 
	blue = N(ix,iy,iz,14);
	DELTA(ix,iy,iz,14) = - blue; 
/* still seed diffusion with scheme above in the -z
   direction. Let's try to kill it by reinjecting red in the opposite 
   direction */
	iz=Nz-1;               /* inject at iz = Nz-1; */
	red = N(ix,iy,iz,4);
	DELTA(ix,iy,iz,4) = red; 
	red = N(ix,iy,iz,5);
	DELTA(ix,iy,iz,5) = red; 
	red = N(ix,iy,iz,7);
	DELTA(ix,iy,iz,7) = red;	
	red = N(ix,iy,iz,8);
	DELTA(ix,iy,iz,8) = red; 
	red = N(ix,iy,iz,17);
	DELTA(ix,iy,iz,17) = red;
      }
}


void ColorRed(void)
{
  int ix, iy, iz, j;
  float red, blue;
  

   for(iz=0;iz<Nz;iz++)
     for(iy=0;iy<Ny;iy++)
       for(ix=0;ix<Nx;ix++)
	 for(j=0;j<19;j++)
	   {
	     red = N(ix,iy,iz,j);
	     DELTA(ix,iy,iz,j) = red;
	   }
} 
	
void WriteCAtZ(char* szOut, int time, int iz)
/*-----------------------------------------------------*
  writes concentration of blues, Cr,
  to ascii file at a distance iz from
  the injection. Cb is blue/(red+blue)
  the way the routine is setup new values of
  the concentration will be appended
 *-----------------------------------------------------*/
{
  char szC[128]; 
  int ix, iy, j;
  float blue, rho, Cb;

  static FILE* fPtrOut; /* to overwrite: remove cast static */

  
 
  if(fPtrOut == NULL)
    {
      sprintf(szC,"%s.con",szOut);
      if(!(fPtrOut=fopen(szC,"w")))
	fprintf(stderr,"%s:%d cannot open file %s\n",__FILE__,
	      __LINE__, szC);
    }


/* average over the plane perpendicular to the flow direction */  
  blue = rho = 0.0;
  for(iy=0;iy<Ny;iy++)
    for(ix=0;ix<Nx;ix++)
      {
	for(j = 0; j < 19; j++)
	  {
	    rho += N(ix,iy,iz,j);
	    blue += 0.5*(N(ix,iy,iz,j)-DELTA(ix,iy,iz,j));
	  }
      }

/* write the concentration to a file */
  Cb = blue/rho;
  fprintf(fPtrOut,"%d\t %f\n",time, Cb); 
/*  fprintf(stderr,"%d\t %f\n",time, Cb);  */
}
  
void WriteC(char* szOut, int time)
/*-----------------------------------------------------*
  writes concentration Cb  of blue
  to a binary file ,where Cb = blue/(red+blue)
  the way the routine is setup new values of
  the concentration will be appended.
  binary file format:
         - time
	 - Nz 
         - Nz Cb values
  NB. if time < 0 then flush 
 *-----------------------------------------------------*/
{
  char szC[128]; 
  int ix, iy, iz,  j;
  float blue, rho, Cb;

  static FILE* fPtrOut; /* to overwrite: remove cast static */

  if(time < 0 && fPtrOut != NULL)
    {
      fclose(fPtrOut);
      fPtrOut=NULL;
      return;
    }
  if(fPtrOut == NULL)
    {
      sprintf(szC,"%s.con",szOut);
      if(!(fPtrOut=fopen(szC,"w")))
	fprintf(stderr,"%s:%d cannot open file %s\n",__FILE__,
	      __LINE__, szC);
      fprintf(stderr,"concentrations of blue will be written to: %s\n",szC);
    }

  fwrite((char *) &Nz, sizeof(int), 1, fPtrOut);   /* - write Nz       */
/* average over the plane perpendicular to the flow direction */ 
  for (iz=0;iz<Nz;iz++) 
    {
      blue = rho = Cb = 0.0;
      for(iy=0;iy<Ny;iy++)
	for(ix=0;ix<Nx;ix++)
	  {
	    for(j = 0; j < 19; j++)
	      {
		rho += N(ix,iy,iz,j);
		blue += 0.5*(N(ix,iy,iz,j)-DELTA(ix,iy,iz,j));
	      }
	  }
      if(rho > 0.0)
	Cb = blue/rho;
      fwrite((char *) &Cb, sizeof(float), 1, fPtrOut);/* - write the concentration */
    }

}






  
void MovieDump(char* szMovie)
/*-----------------------------------------------------*
  writes concentration Cb  of blue
  to a binary character file ,where Cb = blue/(red+blue)
  the way the routine is setup new values of
  the concentration will be appended.
  binary file format:
  Nx * Ny * Nz * record char in movie file
  NB. if record < 0 then flush 
  
  szMovie: movie file name
  
 *-----------------------------------------------------*/
{
  char szC[128], szH[128]; 
  int ix, iy, iz,  j;
  float blue, rho, Cb;
  char color;
  static int steps;
  static FILE *fPtrMovie, *fPtrHeader; 
  /* to overwrite: remove cast static */

/*  if(fPtrMovie != NULL)
    {
      fclose(fPtrMovie);
      fclose(fPtrHeader);
      fPtrMovie=NULL;
      return;
    } */
  if(fPtrMovie == NULL)
    {
      sprintf(szC,"%s.mv",szMovie);
      sprintf(szH,"%s.mv.h",szMovie);      
      if(!(fPtrMovie=fopen(szC,"w")))
	fprintf(stderr,"%s:%d cannot open file %s\n",__FILE__,
	      __LINE__, szC);
      if(!(fPtrHeader=fopen(szH,"w")))
	fprintf(stderr,"%s:%d cannot open file %s\n",__FILE__,
	      __LINE__, szH);
      fprintf(stderr,"movie file is: %s\n",szC);
      fprintf(fPtrHeader,"in=%s\n",szC);
      fprintf(fPtrHeader,"n1=%d\n",Nx);
      fprintf(fPtrHeader,"n2=%d\n",Nz);
    }
  for (iz=0;iz<Nz;iz++) 
    {
      for(iy=0;iy<Ny;iy++)
	for(ix=0;ix<Nx;ix++)
	  {
	    if(!SOLID(ix,iy,iz))
	      {
		rho = blue = 0.0;
		for(j = 0; j < 19; j++)
		  {
		    rho += N(ix,iy,iz,j);
		    blue += 0.5*(N(ix,iy,iz,j)-DELTA(ix,iy,iz,j));
		  }
		if(rho > 0.0)
		  Cb = blue/rho;
		else 
		  Cb=0.0;
		if(Cb > .90)
		  color = 110;
/*		else if(Cb > .5 && Cb <= 0.90)
		  color = 120;
		else if(Cb > 0.1 && Cb <= 0.50)
		  color = 130; */
		else
		  color = 120;
	      }
	    else
	      color = 130;
	    
/* assign colors	to Cb */
/*	    fprintf(stderr,"%d\n",color); */
	    fwrite((char *) &color, sizeof(char), 1, fPtrMovie);
	  }
    }
}












