#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif
#ifndef PI
#define PI 3.14
#endif

void InitLat(float redprob, 
	     float density, 
	     float fluctuation,
	     char inittype,
	     float k)
{
/* ---------------------------------------* 
 * initialize lattice
 * need to add possibility of 
 * non zero velocities
 * ---------------------------------------*/

  int i, j;
  int ix, iy, iz;
  int ixwave;
  float fRandBlue[19], fRandRed[19];
  float redtotal, bluetotal;
  float Cmax, localdensity;
  float cmsq;

  cmsq=3.0;  
  Cmax=1.0;
  redtotal=bluetotal=0.0;
  

  for(iz = 0; iz < Nz; iz++)
    for(iy = 0; iy < Ny; iy++)
      for(ix = 0; ix < Nx; ix++)
	{

	  if(0 == SOLID(ix,iy,iz))  /* inside void space */
	    {
/* switch to options */

	      switch(inittype)
		{

		case 'n':

		  for(j=0; j<19; j++) /* assume one rest particle */
		    {
		      fRandRed[j]   = 1+(2.0*ran1(&seed)-1.0)*fluctuation;
		      fRandRed[j]  *= redprob;
		      fRandBlue[j]  = 1+(2.0*ran1(&seed)-1.0)*fluctuation;
		      fRandBlue[j] *= (1.0 - redprob);
		    }

		  for(j =0;j<12; j++)  /* diagonals */
		    {
		      N(ix,iy,iz,j) = (fRandRed[j]+fRandBlue[j])*density/36.0;
		      DELTA(ix,iy,iz,j) = (fRandRed[j]-fRandBlue[j])*density/36.0;
		    }
		
		  for(j =12;j<18; j++) /* principal axis */                      
		    {
		      N(ix,iy,iz,j) = (fRandRed[j]+fRandBlue[j])*density/18.0; 
		      DELTA(ix,iy,iz,j) = (fRandRed[j]-fRandBlue[j])*density/18.0; 
		    }
		  N(ix,iy,iz,18)=(fRandRed[j]+fRandBlue[j])*density/3.0; 
		  DELTA(ix,iy,iz,18)=(fRandRed[j]-fRandBlue[j])*density/3.0; 
		  break;

                case 'd':                         /* diffusion */
/*--------------------------------------------------------------------*
set blue density to zero and red to:
		     C(x,0)=0.5*Cmax*(sin(k*2.0*PI*ix/Nx)+1.0) 
		     where C=red/(red+blue) 
initialisation has zero velocity		     
*---------------------------------------------------------------------*/
		  {
		    localdensity=0.5*Cmax*(sin(k*2.0*PI*ix/Nx)+1.0) ;
		    for(j =0;j<12; j++)  /* diagonals */
		      {
			N(ix,iy,iz,j)=1.0/36.0;
			DELTA(ix,iy,iz,j) = (2*localdensity-1.0)/36.0;
		      }
		    for(j =12;j<18; j++) /* principal axis */                      
		      {
			N(ix,iy,iz,j) = 1.0/18.0;
			DELTA(ix,iy,iz,j) = (2*localdensity-1.0)/18.0; 
		      }
		    N(ix,iy,iz,18)= 1/3.0;
		    DELTA(ix,iy,iz,j) =  (2*localdensity - 1.0)/3.0; 
		  }
		  break;
		}
	    }
	  else /* if solid */
	    for(j=0;j<19;j++)
	      N(ix,iy,iz,j)=DELTA(ix,iy,iz,j) = 0.0;

	  for(j=0;j<19;j++)
	    {
	      redtotal  += 0.5*(N(ix,iy,iz,j)+DELTA(ix,iy,iz,j));
	      bluetotal += 0.5*(N(ix,iy,iz,j)-DELTA(ix,iy,iz,j));
	    }
	}
  fprintf(stderr,"redtotal = %f\t bluetotal = %f\n",redtotal, bluetotal);

}
   

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

void FindSolidSurface(void)
{
/*--------------------------------------------------------------------*
   find surface of solid and lable.
   To be consistent with John's format, i use lables:
   void     0
   surface  1
   interior 5 (100 | 001)
   algorithm: initialize fluid in all lattice directions. 
              Propagate and check if solid has non zero directions.
 *--------------------------------------------------------------------*/

  int i, j;
  int ix, iy, iz;
  int nSurfaceCount, nVoidCount;

  for(iz=0;iz<Nz;iz++)  /* initialize fluid */
    for(iy=0;iy<Ny;iy++)
      for(ix=0;ix<Nx;ix++)
	{
	  if(0 == SOLID(ix,iy,iz))
	    for(j = 0; j<18;j++)
	      N(ix,iy,iz,j) = 1.0;
	  else
	    for(j = 0; j<18;j++)
	      N(ix,iy,iz,j) = 0.0;
	}

  Propagate(n);               /* propagate one time step */

  for(i=0, nSurfaceCount = 0, nVoidCount = 0; i < Nx*Ny*Nz; i++)      
    {
      if(solid[i] & 1)        /* is it solid? */
	if(0 == n[i])         /*is it an interior site? */
	 solid[i] |= 4;
	else
	  {
	    solid[i] = 1;       /* surface site */
	    nSurfaceCount++;
	  }
      else
	nVoidCount++;
    }
  fprintf(stderr,"Found %d surface nodes and %d void nodes.\n", 
	  nSurfaceCount, nVoidCount);
}

  
    

  



