#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif


void Relaxate(void)
{
  int ix, iy, iz;
  int i,j;
  int alpha, beta;
  float drho, rho, v[3];
  float NEq[19], DeltaEq[19];
  float NOld[19], DeltaOld[19];
  float G, cmsq;


  G=4.5;
  cmsq=3.0;

  for(ix=0;ix<Nx;ix++)
    for(iy=0;iy<Ny;iy++)
      for(iz=0;iz<Nz;iz++)
	{
/*----------------------------------------------------------------*
  Apply relaxation rule in voids and
  bounce back rules on and in solids (boundaries),
  that is no differentation between
  interior nodes and boundary of the
  solid
 *----------------------------------------------------------------*/
	  if(!SOLID(ix,iy,iz)) /* if void */
	    {
	      drho=rho=0.0;
	      v[0]=v[1]=v[2]=0.0;

/* compute rho, vx,vy,vz */
	      for(j=0;j<19;j++)
		{
		  NOld[j] = N(ix,iy,iz,j);
		  DeltaOld[j] = DELTA(ix,iy,iz,j);
		  rho += N(ix,iy,iz,j);
		  drho += DELTA(ix,iy,iz,j);
		  for(alpha=0;alpha<3;alpha++)
		    v[alpha] += C(j,alpha)*N(ix,iy,iz,j);
		}
	      if(rho != 0.0)
		for(alpha=0;alpha<3;alpha++)
		  v[alpha] /= rho;
	      else
		for(alpha=0;alpha<3;alpha++)
		  v[alpha] = 0.0;

/*---------------------------------------------------------------*
   compute the equilibrium distribution for
   both the particles and the concentration:
*----------------------------------------------------------------*/
	      for(j=0;j<19;j++)
		{
		  NEq[j] = 1.0;
		  DeltaEq[j] = 1.0;
		  for(alpha=0;alpha<3;alpha++)
		    {
		      NEq[j] += cmsq*C(j,alpha)*v[alpha];
		      DeltaEq[j] += cmsq*C(j,alpha)*v[alpha];
		      for(beta=0;beta<3;beta++)
			NEq[j] += G*Q(j,alpha,beta)*v[alpha]*v[beta];
		    }
		}
/* correct for the model, here only D3Q19 */
	      for(j=0;j<12;j++)             /* for diagonal */
		{
		  NEq[j] *= (rho/36.0);
		  DeltaEq[j] *= (drho/36);
		}
	      for(j=12;j<18;j++)            /* for principal axes */
		{
		  NEq[j] *= (rho/18.0);
		  DeltaEq[j] *= (drho/18.0);
		}	    
	      NEq[18] *= (rho/3.0);         /* for rest particle */
	      DeltaEq[18] *= (drho/3.0);

/*----------------------------------------------------------------*
 * relaxation rule 
 *----------------------------------------------------------------*/	

	      for(i=0;i<19;i++)
		{
		  N(ix,iy,iz,i)=NOld[i]+lambda_Nu*(NOld[i]-NEq[i]);
		  DELTA(ix,iy,iz,i)=DeltaOld[i]+
		    lambda_D*(DeltaOld[i]-DeltaEq[i]);
		}
	    }
/*-----------------------------------------------------------------*
 * bounce back
 *-----------------------------------------------------------------*/

	  else /* then bounce back */
	    {
	      for(j=0;j<19;j++)
		{
		  NOld[j] = N(ix,iy,iz,j);
		  DeltaOld[j]=DELTA(ix,iy,iz,j);
		}
/* for diagonal particles */
	      for( i = 0; i < 6; i++)
		{
		  N(ix,iy,iz,i) = NOld[i+6];
		  N(ix,iy,iz,i+6) = NOld[i];
		  DELTA(ix,iy,iz,i) = DeltaOld[i+6];
		  DELTA(ix,iy,iz,i+6) = DeltaOld[i];
		}
/* for principal axes */ 
	      for(i = 12; i < 15; i++)
		{
		  N(ix,iy,iz,i) = NOld[i+3];
		  N(ix,iy,iz,i+3) = NOld[i];
		  DELTA(ix,iy,iz,i) = DeltaOld[i+3];
		  DELTA(ix,iy,iz,i+3) = DeltaOld[i];
		}

	    }
	}
}




