#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif

float Diffusion(void)
{
/*--------------------------------------------------------------*
 * compute the area for a concentration distribution:
 * |C(x,t) -1/2|=1/2*e^(-Dk^2t)*|sin(k(x-ut)| 
 * area computed by simple Riemann summation:
 * A(t) = Sum(|C(x,t) - 1/2|)
 *--------------------------------------------------------------*/
  int ix, iy, iz;
  int i;
  float Cr, rho, red;
  float A;

  A=0.0;
  for(iz=0;iz<Nz;iz++)
    for(iy=0;iy<Ny;iy++)
      for(ix=0;ix<Nx;ix++)
	{
	  red = rho = 0.0;
	  for(i=0;i<19;i++)
	    {
	      rho += N(ix,iy,iz,i);
	      red += 0.5*(N(ix,iy,iz,i)+DELTA(ix,iy,iz,i));
	    }
	  Cr=red/rho;
	  A += fabs(Cr-0.5);
	}  
  
  A /= (float) (Ny*Nz);
  return A;
}
	      
float Viscosity(float umax)
{
/*--------------------------------------------------------------*
 * compute the area of velocity distribution:
 * |v(y,t) -1/2|=1/2*e^(-Dk^2t)*|sin(k(y)| 
 * area computed by simple Riemann summation:
 * A(t) = Sum(|v(x,t) - 1/2*vmax|)
 *--------------------------------------------------------------*/
  int ix, iy, iz;
  int i;
  float rho, w;
  float A;

  A=0.0;
  for(iz=0;iz<Nz;iz++)
    for(ix=0;ix<Nx;ix++)
      for(iy=0;iy<Ny;iy++)
	{
	  w=rho=0;;
	  for(i=0;i<19;i++)
	    {
	      rho += N(ix,iy,iz,i);
	      w += N(ix,iy,iz,i)*C(i,2);
	    }
	  w /= rho;
	  A += fabs(w);
	}  
  A /= (float) (Nx*Nz);

  return A;
}
	      
	  
