/*<<<<<<<<<<<<<<<<<<<DRIVER for BGK>>>>>>>>>>>>>>>>>*/
#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif

void main()
/*---------------------------------------------------------------------*
   driver to run MISCBGK
   commands read from script file. Each line in the script file 
   schould begin with a prefix listed below, followed by a single
   space.
    prefixs are:
    '.':           indicates the end of the scrpt file
    'f':           output file name
    'r':           seed random generator
    'm':           select FCHC model and set the direction vectors
    's':           read solid and set universe
    'i':           initialize lattice
    'c':           do experiment for flow through solid

    'p':           read two parameters lambda_Nu, lambda_D
    'd':           diffussion
    'g':           gravity
    'n':           color red
    
 *------------------------------------------a---------------------------*/   

{
  char prefix;
  char szModel[128];
  FILE* fPtrSolid;
  BOOL bIsParam = FALSE;

  int alpha, i;
  int ix, iy, iz;
  float redprob, fluctuation;
  float vx, vy, vz, k;
  char inittype; 

 /* read defaults values */
  fprintf(stderr,"*************************************************************\n");
  fprintf(stderr,"* miscible 3D Boltzmann (BGK) code                       *\n");
  fprintf(stderr,"* after Flekkoy's method                                   *\n");
  fprintf(stderr,"* Author: Olav van Genabeek                                 *\n");
  fprintf(stderr,"*         Department of Earth Plan. and Atmos. Sciences     *\n");
  fprintf(stderr,"*         email: olav@segovia.mit.edu                       *\n");
  fprintf(stderr,"* Cambridge, March, 1997                                    *\n");
  fprintf(stderr,"*************************************************************\n");
  seed = (long) time(NULL);                           /* seed random gen */
  fprintf(stderr,"current date: %s",ctime(&seed));
  seed = -seed;

  scanf("%c",&prefix);
  while(prefix != '.')
    {
      switch(prefix)
	{
	case 'm':
	  scanf(" %s", szModel);
	    if(!strcmp("D3Q19",szModel))
	      {
		fprintf(stderr,"using %s model\n",szModel);
		SetD3Q19();
/*		for(i=0;i<19;i++)
		  fprintf(stderr,"%5d%3d%3d%3d\n",
			  i,C(i,0),C(i,1),C(i,2)); */

	      }
	  break;
	case 'r':          /* seed random generator */
	  scanf(" %ld", &seed);
	  break;
	case 'f':          /* set finame for future outputs */
	  scanf(" %s", szOut);
	  break;
	case 'u':          /* set universe, cannot be used after switch s!! */
	  scanf(" %d %d %d",&Nx, &Ny, &Nz);
	  fprintf(stderr,"dimensions %dx%dx%d\n",Nx,Ny,Nz);
	  InitPropagate();
	  SetUniverse(Nx,Ny,Nz);
	  break;
	case 'p':          /* set relaxation constants: lambda_D, lambda_Nu */
	  scanf(" %f %f", &lambda_Nu, &lambda_D);
	  fprintf(stderr,"viscosity:   %f\n", (-1/3.)*(1/lambda_Nu+.5));
	  fprintf(stderr,"diffusivity: %f\n", (-1/3.)*(1/lambda_D+.5));
	  bIsParam=TRUE;
	  break;
	case 'g':
	  scanf(" %f", &gravity);
	  fprintf(stderr,"gravity: %f\n",gravity);
	  break;
	case 'n':
	  ColorRed();
	  break;
	case 's':          /* read solid */
	  scanf(" %s", szSolid);
	  if(!(fPtrSolid=fopen(szSolid, "r")))
	    {
	      fprintf(stderr,"%s:%d cannot open file: %s\n",__FILE__,
		      __LINE__, szSolid);
	      exit(1);
	    } 
	  fread( (char *) &Nx, sizeof(int), 1, fPtrSolid);
	  fread( (char *) &Ny, sizeof(int), 1, fPtrSolid);
	  fread( (char *) &Nz, sizeof(int), 1, fPtrSolid);
	  fprintf(stderr,"dimensions %dx%dx%d\n",Nx,Ny,Nz);
	  InitPropagate(); 
	  SetUniverse(Nx, Ny, Nz);
	  if(!(fread( (char *) solid, sizeof(char), Nx*Ny*Nz,
		     fPtrSolid)))
	    {
	      fprintf(stderr,"%s:%d cannot read file: %s", __FILE__,
		      __LINE__, szSolid);
	      exit(1);
	    }
	  fprintf(stderr,"reading solid file: %s ",szSolid);
	  fprintf(stderr,"...Ok\n");
	  fclose(fPtrSolid);
	  break;
	case 'i':          /* initializing lattice, careful: universe should be set!! */
	  {

	    
	    fprintf(stderr,"random generator seeded with: %ld\n",seed);
	    scanf(" %c %f %f",&inittype, &density, &redprob);
	    fluctuation=0.0;
	    fprintf(stderr,"density:         %f\n",density);
	    fprintf(stderr,"red probability: %f\n",redprob);
	    printf("type: %c\n",inittype);
	    k=0.0;
	    if(inittype == 'd')
	      {
		scanf("%f",&k);
		fprintf(stderr,"wave number: %f\n",k); 
	      }
	    fprintf(stderr,"initializing lattice \n");
	    InitLat(redprob, density, 0, inittype,k);
	    TestCons(); 
	    fprintf(stderr,"\n");
	    break;
	  }
	case 'c':
	  {
/* read: parameters: lambda_Nu and lambda_D
   number of frames, steps */
	    int iframe, frames, istep, steps;
	    char mode, writemode;


	    if(bIsParam == FALSE)
	      {
		fprintf(stderr,"need parameters\n");
		break;
	      }
	    else
	      {
		scanf(" %c %d %d %c",&mode, &frames, &steps, &writemode);	    
		fprintf(stderr,"frames=%d steps=%d\n",frames, steps);

/* 
   mode == v: write velocities
        == c: write concentrations
	== m: create movie file
  should combine latter two modes to one in near future
*/
		switch(mode)
		  {
		  case 'v':
		    fprintf(stderr,"equilibrating...");
		    for(iframe=0;iframe<frames;iframe++) 
		      {
			for(istep=0;istep<steps;istep++)
			  {
			    Propagate(n);
			    Propagate(delta);
			    Relaxate();  
			    AddGravity(density, gravity);
			  }
			TestCons();   
		      }
/*		    WriteVel(szOut,writemode);  */
		    fprintf(stderr,"done\n");
		    break;
		  case 'c':		      
		    {
		      for(iframe=0;iframe<frames;iframe++) 
			{
			  for(istep=0;istep<steps;istep++)
			    {
			      Propagate(n);
			      Propagate(delta);
			      Relaxate();  
			      Inject();
			      AddGravity(density, gravity);
			    }
			  WriteC(szOut,iframe*istep);  			  
			  TestCons();   
			}
		      WriteC(szOut,-1); /* flush the file pointer */
		    }
		    break;                                       /* end of writemode switch */
		  case 'm':
		    fprintf(stderr,"making movie\n");
		    for(iframe=0;iframe<frames;iframe++) 
		      {
			fprintf(stderr,"frame: %d\n",iframe);
			for(istep=0;istep<steps;istep++)
			  {
			    Propagate(n);
			    Propagate(delta);
			    Relaxate();  
			    Inject();
			    AddGravity(density, gravity);
			  }
			  MovieDump(szOut);
			TestCons();   
		      }
/*		    MovieDump(szOut,-1); */ /* flush the file pointer */ 
		    break;                                       /* end of movie mode switch */	
		  }                                               
	      }
	  }
	  break;                                                 /* end of upper 'c' switch */

	case 'd':                                                /* begin of test switch    */
	  {
/* read: parameters: lambda_Nu and lambda_D
   number of frames, steps */
	    int iframe, frames, istep, steps;
	    char mode;
	    float A0, wavenr;
	    fprintf(stderr,"--------------------------------------------------\n");
/* d: compute diffusion
   v: compute kinematic viscosity */
	    switch(inittype)
	      {
	      case 'd':
		fprintf(stderr,"computation of diffusion coefficient\n");
		A0=Diffusion();
		printf("%f\n",A0);
		break;
	      }
	    fprintf(stderr,"--------------------------------------------------\n");
	    fprintf(stderr,"dimensionless wave nr.:\t%d\n",(int) k);

	    if(bIsParam == FALSE)
	      {
		fprintf(stderr,"need parameters\n");
		break;
	      }
	    else
	      {
		scanf(" %c %d %d",&mode, &frames, &steps);	 
		fprintf(stderr,"frames:\t\t\t:%d \n",frames);
		fprintf(stderr,"steps:\t\t\t:%d \n",steps);
		fprintf(stderr,"--------------------------------------------------\n");
		fprintf(stderr,"t\tk^2 t\t\tln(A(0)/(A(t)))\n");
/*			WriteVel(szOut,'t'); */
		for(iframe=0;iframe<frames;iframe++) 
		  {
		    for(istep=0;istep<steps;istep++)
		      {
			Propagate(n);
			Propagate(delta);
			Relaxate();  
		      }
/*		    TestCons();  */
/* !!!!NB !!!! 
   inspect initlat.c to find out what wavenr is: 
   wavenr = 2 Pi n / Nx, where k is number of waves 
*/
		    if(inittype == 'd')		
		      {    
			wavenr = 2*PI/Nx * k;
			fprintf(stderr,"%d\t%f\t%f\n",(iframe+1)*istep, 
			      (iframe+1)*istep*(wavenr*wavenr),
				log(A0/Diffusion()));
		      } 
		  }
 		fprintf(stderr,"--------------------------------------------------\n");
	      }
	    break; 

	  } /* end of param */

	}
      scanf("%c",&prefix);
    }

  CleanUniverse();
  free(map); /*!!!!! needs to be included in both set 
	       and clean universe routines !!!!!*/
}










