#ifndef BGK_H 
#define BGK_H
#include "bgk.h"
#endif

#define MASS(ix,iy,iz) mass[((iz)*Ny+(iy))*Nx+(ix)]
#define PX(ix,iy,iz)   px[((iz)*Ny+(iy))*Nx+(ix)]
#define PY(ix,iy,iz)   py[((iz)*Ny+(iy))*Nx+(ix)]
#define PZ(ix,iy,iz)   pz[((iz)*Ny+(iy))*Nx+(ix)]

/*----------------------------------------------------------*
  writes velocties to a file 'szOut'.vel
 *----------------------------------------------------------*/

void WriteVel(char* szOut, char mode)
{
  int ix,iy,iz,i;
  int alpha;
  char szVel[128];
  float v[3];

  static BOOL bIsAllocated = FALSE;
  static FILE* fPtrOut = NULL;
  static float* mass =NULL;
  static float* px=NULL;
  static float* py=NULL;
  static float* pz=NULL;

  float tmass;
  float tpx;
  float tpy;
  float tpz;

  switch(mode)
    {
    case 'a': /* average */
      if(bIsAllocated == FALSE)
	{
	  if(!(mass=(float*) calloc(Nx*Ny*Nz,sizeof(float))))
	    {
	      fprintf(stderr,"%s:%d allocation error\n",__FILE__,__LINE__);
	      exit(1);
	    }
	  if(!(px=(float*) calloc(Nx*Ny*Nz,sizeof(float))))
	    {
	      fprintf(stderr,"%s:%d allocation error\n",__FILE__,__LINE__);
	      exit(1);
	    }
	  if(!(py=(float*) calloc(Nx*Ny*Nz,sizeof(float))))
	    {
	      fprintf(stderr,"%s:%d allocation error\n",__FILE__,__LINE__);
	      exit(1);
	    }
	  if(!(pz=(float*) calloc(Nx*Ny*Nz,sizeof(float))))
	    {
	      fprintf(stderr,"%s:%d allocation error\n",__FILE__,__LINE__);
	      exit(1);
	    }
	  bIsAllocated = TRUE;
	}
      for(iz=0;iz<Nz;iz++)
	for(iy=0;iy<Ny;iy++)
	  for(ix=0;ix<Nx;ix++)
	    for(i=0;i<18;i++)
	      {
		MASS(ix,iy,iz) += N(ix,iy,iz,i);
		PX(ix,ix,iz) += C(i,0)*N(ix,iy,iz,i);
		PY(ix,ix,iz) += C(i,1)*N(ix,iy,iz,i);
		PZ(ix,ix,iz) += C(i,2)*N(ix,iy,iz,i);
	      }
      break;
    case 'w':
/* write velocities to a binary file with Nx*Ny*Nz velocities
   int the 3 directions, wtih dimesnsions Nx, Ny, Nz reps.
   Here is Nx the fast and, .. Nz the slow dimension 
   file format: _ Nx,
                - Ny,
                - Nz,
		- vx(ix,iy,iz), vy(ix,iy,iz), vz(ix,iy, iz) for all ix, iy, iz 
*/

      if(bIsAllocated == TRUE)
	{
	  sprintf(szVel,"%s.avel",szOut);
          fprintf(stderr, "saving averaged velocity field in file: %s\n",szVel);
	  if(!(fPtrOut=fopen(szVel,"w")))
	    {
	      fprintf(stderr,"%s:%d cannot open file %s\n",__FILE__,
		      __LINE__, szVel);
	    }
	  fwrite((char *) &Nx, sizeof(int), 1, fPtrOut);
	  fwrite((char *) &Ny, sizeof(int), 1, fPtrOut);
	  fwrite((char *) &Nz, sizeof(int), 1, fPtrOut);

	  for(iz=0;iz<Nz;iz++)
	    for(iy=0;iy<Ny;iy++)
	      for(ix=0;ix<Nx;ix++)
		{
		  if(MASS(ix,iy,iz) == 0)
		      v[0]=v[1]=v[2]=0.0;
		  else
		    {
		      v[0]=PX(ix,iy,iz)/MASS(ix,iy,iz);
		      v[1]=PY(ix,iy,iz)/MASS(ix,iy,iz);
		      v[2]=PZ(ix,iy,iz)/MASS(ix,iy,iz);
		    }
		  fwrite((char *) v, sizeof(float), 3, fPtrOut);
		}
	  free(mass);
	  free(px);
	  free(py);
	  free(pz);
	  fclose(fPtrOut);
	  bIsAllocated = FALSE;
	}
      break;
    case 't':
	  sprintf(szVel,"%s.vel",szOut);
          fprintf(stderr, "saving velocity field in file: %s\n",szVel);
	  if(!(fPtrOut=fopen(szVel,"w")))
	    {
	      fprintf(stderr,"%s:%d cannot open file %s\n",__FILE__,
		      __LINE__, szVel);
	    }
	  fwrite((char *) &Nx, sizeof(int), 1, fPtrOut);
	  fwrite((char *) &Ny, sizeof(int), 1, fPtrOut);
	  fwrite((char *) &Nz, sizeof(int), 1, fPtrOut);

	  for(iz=0;iz<Nz;iz++)
	    for(iy=0;iy<Ny;iy++)
	      for(ix=0;ix<Nx;ix++)
		{
		  tmass= 0;
		  v[0]=v[1]=v[2]=0.0;
		  for(i=0;i<19;i++)
		    {
		      tmass += N(ix,iy,iz,i);
		      v[0] += N(ix,iy,iz,i)*C(i,0);
		      v[1] += N(ix,iy,iz,i)*C(i,1);
		      v[2] += N(ix,iy,iz,i)*C(i,2);
		    }
		  if(tmass == 0)
		      v[0]=v[1]=v[2]=0.0;
		  else
		    {
		      v[0] /= tmass;
		      v[1] /= tmass;
		      v[2] /= tmass;
		    }
		  fwrite((char *) v, sizeof(float), 3, fPtrOut);
		}
	  fclose(fPtrOut);
      break;
    }

}
      
