/* 
  $Author: zaleski $
  $Revision$ 
*/
#include <stdio.h>
#include <math.h>
#include <limits.h>

/*  Wrap around boundary conditions */ 

#define WRAP_DN(p,n)	((p >= n) ? (p-n) : p)
#define WRAP_UP(p,n)	(( p < 0) ? (p+n) : p)
#define WRAP_AROUND(p,n)	(( p < 0) ? (p+n) : WRAP_DN(p,n))

/* For parabolas  */

#define NX 64 
#define NY 16
#define WALL_THICKNESS 1
#define NR_OF_GRAINS 0
#define MINIMUM_GRAIN_SIZE 16
#define GRAIN_SIZE_DISPERSION 16

 
/* For interactions 
#define NX 256
#define NY 256
#define WALL_THICKNESS 0
#define NR_OF_GRAINS 0
#define MINIMUM_GRAIN_SIZE 16
#define GRAIN_SIZE_DISPERSION 16
*/


const float BIG=(float) LONG_MAX + 1.;


/* Evaporation parameters */

main()
{
  char *solid;
  char *movie; 
  int i , j, z, nx, ny, nz, seed; 
  int i_rock, j_rock;
  int i1,i2,j1,j2,ii,jj,zx, zy;
  int x_por, y_por, gr_sz_min, gr_sz_disp, nr_of_grains;
  FILE *solidfd, *moviefd;

  nr_of_grains=NR_OF_GRAINS;    
  gr_sz_min=MINIMUM_GRAIN_SIZE;
  gr_sz_disp=GRAIN_SIZE_DISPERSION;  

  seed = time(0);
  srandom(seed);

  nx=NX;
  ny=NY;
  nz = nx*ny;
  solid = (char *) malloc(nz*sizeof(char));
  movie = (char *) malloc(nz*sizeof(char));

  for(z=0;z<nz;z++)
    {
      movie[z] = 0;
      solid[z] = 0;
    }

/* Add walls */

  for (i=0; i< nx; i++)
    for(j=0; j< ny ; j++) 
      {
	z= i + nx*j;
	solid[z] = 0 ;
	if((j< WALL_THICKNESS) || (j> ny - WALL_THICKNESS -1) ) 
	  {
	    solid[z] = 1;
	    movie[z] = 128;
	  }
      }

  /* Add grains */
  
  if(nr_of_grains > 0) 
    {
	  y_por = (ny- 2*WALL_THICKNESS)/nr_of_grains;
	  x_por = (int) ( (sqrt(3.)/2.)* y_por + 0.5);
      
      /* Find coordinates of rock centers i_rock, j_rock  */
      
      for(i_rock=x_por/2-1;i_rock < nx - x_por/2 ; i_rock += x_por)
	{ 
	  for(j_rock= WALL_THICKNESS + y_por/2-1 ;j_rock < ny - WALL_THICKNESS  - y_por/2 ; j_rock+= y_por)
	    { 
	      
	      /* Draw widths  zx and height zy */
	      
	      zx =(int) ((((unsigned long)random())/BIG)*gr_sz_disp 
			 + gr_sz_min );
	      zy =  (int) ((2./sqrt(3.))*zx +0.5 );
	      
	      i1 = i_rock - zx/2;
	      i2 = i_rock + zx/2;
	      j1 = j_rock - zy/2;
	      j2 = j_rock + zy/2;
	      
	      for(ii=i1; ii < i2; ii++)
		for(jj=j1; jj< j2; jj++)
		  {
		    i = WRAP_AROUND(ii,nx);
		    j = WRAP_AROUND(jj,ny);
		    solid[i + nx*j] = 1; 
		    movie[i + nx*j] = 128;
		    
		  }
	    }
	}
    }

  solidfd= fopen("solid","w");
  moviefd= fopen("solidm","w");
  fwrite((char *) solid, sizeof(char), nz*sizeof(char),solidfd);
  fwrite((char *) movie, sizeof(char), nz*sizeof(char),moviefd);
}


